from django.contrib.auth.mixins import LoginRequiredMixin
from django.forms import Form, CharField, IntegerField, BooleanField
from django.views.generic.edit import BaseCreateView
from django.http import HttpResponse
from django.contrib import messages
from django.utils.translation import pgettext

from main.decorators import time_of_function
from main.models import Config, ConfigCleaningTable


class EmailConfigForm(Form):
    smtp_host = CharField()
    smtp_port = IntegerField()
    smtp_user = CharField()
    smtp_password = CharField()
    encryption = CharField()
    send_email = BooleanField(required=False)


class EmailConfigView(LoginRequiredMixin, BaseCreateView):
    model = Config
    form_class = EmailConfigForm
    template_name = 'config/edit_email.html'
    success_url = 'edit_email_config/'

    def get_object(self, queryset=None):
        return Config.objects.first()

    @time_of_function
    def post(self, request, *args, **kwargs):
        if request.user.get_role() != 2:
            return HttpResponse(status=403)
        form = self.get_form()
        if form.is_valid():
            obj = self.get_object()
            for field, value in form.cleaned_data.items():
                setattr(obj, field, value)
            obj.save()
            return HttpResponse(status=200)
        else:
            messages.error(request, pgettext('messages', 'Invalid data'))

            return HttpResponse(form.errors, status=422)


class CleaningConfigForm(Form):
    time_interval = IntegerField(required=False)
    bt1 = BooleanField(required=False)
    bt2 = BooleanField(required=False)
    bt3 = BooleanField(required=False)
    bt4 = BooleanField(required=False)
    bt5 = BooleanField(required=False)
    bt6 = BooleanField(required=False)
    bt7 = BooleanField(required=False)
    bt8 = BooleanField(required=False)
    bt9 = BooleanField(required=False)
    bt10 = BooleanField(required=False)
    bt14 = BooleanField(required=False)
    bt16 = BooleanField(required=False)
    desc = CharField(required=False)
    ip = CharField(required=False)
    vhost = CharField(required=False)
    url = CharField(required=False)
    enable = BooleanField(required=False)


class CleaningConfigView(LoginRequiredMixin, BaseCreateView):
    model = ConfigCleaningTable()
    form_class = CleaningConfigForm
    template_name = 'config/edit_conf_clear_db.html'
    success_url = 'edit_cleaning_config/'

    def get_object(self, id_obj, queryset=None):
        if id_obj is None:
            config = ConfigCleaningTable()
        else:
            config = ConfigCleaningTable.objects.get(id=id_obj)
        return config

    @time_of_function
    def post(self, request, *args, **kwargs):
        if request.user.get_role() != 2:
            return HttpResponse(status=403)
        form = self.get_form()
        row_id = request.POST.get('id', None)
        if form.is_valid():
            obj = self.get_object(id_obj=row_id)
            bt = ''
            for field, value in form.cleaned_data.items():
                if field != 'enable':
                    if value not in (None, '') or request.POST.get('edit', None) is not None:
                        if field == 'time_interval':
                            if value in (None, ''):
                                value = 0
                            elif value < 0:
                                value = 0
                        if 'bt' in field:
                            if value:
                                if bt != '':
                                    bt += ','
                                if field == 'bt1':
                                    bt += '1'
                                elif field == 'bt2':
                                    bt += '2'
                                elif field == 'bt3':
                                    bt += '3'
                                elif field == 'bt4':
                                    bt += '4'
                                elif field == 'bt5':
                                    bt += '5'
                                elif field == 'bt6':
                                    bt += '6'
                                elif field == 'bt7':
                                    bt += '7'
                                elif field == 'bt8':
                                    bt += '8'
                                elif field == 'bt9':
                                    bt += '9'
                                elif field == 'bt10':
                                    bt += '10'
                                elif field == 'bt14':
                                    bt += '14'
                                elif field == 'bt16':
                                    bt += '16'

                            continue
                        if value in (None, ''):
                            value = '*'
                        setattr(obj, field, value)
                else:
                    if request.POST.get('edit', None) is None:
                        setattr(obj, field, value)
            if request.POST.get('edit', None) is not None:
                if bt == '':
                    bt = '*'
                obj.bt = bt
            obj.save()
            return HttpResponse(obj.id, status=200)
        else:
            messages.error(request, pgettext('messages', 'Invalid data'))

            return HttpResponse(form.errors, status=422)
