import logging
import time

from memory_profiler import memory_usage, profile
from django.conf import settings


DEBUG_MEMORY = settings.DEBUG_MEMORY & settings.DEBUG

logger = logging.getLogger(__name__)


class MemoryProfilerMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if DEBUG_MEMORY:
            start_mem = memory_usage()[0]
            start_time = time.time()

            response = self.get_response(request)

            end_time = time.time()
            end_mem = memory_usage()[0]

            logger.info(f"Request: {request}")
            logger.info(f"Memory used: {end_mem - start_mem} MiB")
            logger.info(f"Time taken: {end_time - start_time} seconds")

        else:
            response = self.get_response(request)

        return response


