""" Django loggers settings """
from pathlib import Path

import os

try:
    from cabinet.settings import DEBUG
except (ImportError, NameError):
    DEBUG = False

_report_loglevel = 'DEBUG' if DEBUG else 'INFO'
_log_folder = '/var/log/uwsgi/cabinet/' if not os.getenv('DEV') else str(Path.cwd()) + '/log/'

Path(_log_folder).mkdir(parents=True, exist_ok=True)

if DEBUG:
    log_level = 'DEBUG'
else:
    log_level = 'INFO'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'console': {
            'format': '%(asctime)s %(levelname)-6s %(message)s',
            'datefmt': '%d-%m-%Y %H:%M:%S',
        },
        'file': {
            'format': '%(asctime)s %(levelname)-6s %(name)s.%(funcName)s %(message)s',
            'datefmt': '%d-%m-%Y %H:%M:%S',
        }
    },

    'handlers': {
        'file': {
            'class': 'logging.FileHandler',
            'filename': _log_folder + 'app.log',
        },
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'console'
        },
        'file_report': {
            'level': log_level,
            'class': 'logging.FileHandler',
            'filename': _log_folder + 'report.log',
            'formatter': 'file',
        },
        'file_user': {
            'level': log_level,
            'class': 'logging.FileHandler',
            'filename': _log_folder + 'user.log',
            'formatter': 'file',
        },
    },

    'loggers': {
        'django': {
            'handlers': ['file'],
            'level': 'ERROR',
            'propagate': True,
        },
        '': {
            'level': log_level,
            'handlers': ['console']
        },
        'asyncio': {
            'level': 'WARNING',
        },
        'urllib3': {
            'level': 'WARNING',
        },
        'report': {
            'handlers': ['file_report'],
            'level': log_level,
            'propagate': True,
        },
        'user': {
            'handlers': ['file_user'],
            'level': log_level,
            'propagate': True,
        },
    },
}
