"""
BT settings for report and charts
"""
import os
import tempfile
from dataclasses import dataclass
from collections import OrderedDict
from pathlib import Path


@dataclass(frozen=True)
class BT:
    number: int
    description: str
    show_in_report: bool = True
    show_in_charts: bool = True


@dataclass(frozen=True)
class BTChart:
    description: str
    bt_numbers: list
    color: str = '#ffffff'


BT_LIST = [
    BT(
        number=1,
        description='Signature Analysis'
    ),
    BT(
        number=2,
        description='Signature Analysis'
    ),
    BT(
        number=3,
        description='Nemesida AI MLA'
    ),
    BT(
        number=4,
        description='Antivirus'
    ),
    BT(
        number=5,
        description='Internal error',
        show_in_report=False
    ),
    BT(
        number=6,
        description='Auto-blocked IP',
        show_in_report=False
    ),
    BT(
        number=7,
        description='Brute-force detection',

    ),
    BT(
        number=8,
        description='Nemesida AI MLC',
    ),
    BT(
        number=9,
        description='Flood detection',
    ),
    BT(
        number=10,
        description='DDoS detection',
    ),
    BT(
        number=14,
        description='OpenAPI inspection'
    ),
    BT(
        number=16,
        description='Blocked IP'
    ),

]

CHART_LIST = [
    BTChart(
        description='Signature Analysis',
        bt_numbers=[1, 2],
        color='#1a5287'
    ),
    # todo fill CHART_LIST
]

# SETTINGS FOR REPORTS:
# Title list
#
# Counting attacks, by the bt keyword/number in the description field
# format to add new line: (db_description/bt_number, report_description)
TITLE_BT_DESCRIPTION_COUNTER = OrderedDict([
    ('SQLi', 'SQL Injection'),
    ('XSS', 'Cross-site scripting'),
    ('Virus', 'Virus'),
])

# format: (bt num , description for fields)
TITLE_BT_COUNTERS = OrderedDict([
    (7, 'Brute-force'),
    (9, 'Flood'),
    (10, 'DDoS'),
])
# after that will be count 'other' field in report

# the data configured below will go as a separate block in the report
# format: (description for fields, [bt nums])
TITLE_BT_EXTRA_COUNTERS = OrderedDict([
    ('Blocked by Signature Analysis Module', [1, 2]),
    ('Blocked by Machine Learning Module', [3, 8, 9]),
])

BT_REPORT_EXCLUDE_NUMBERS = [bt.number for bt in BT_LIST if not bt.show_in_report]

# making BT_DICT
BT_DICT = {}
for obj in BT_LIST:
    BT_DICT[obj.number] = obj

FIELDS_FOR_REPOST = ['vhost', 'timestamp', 'mz', 'method', 'ip', 'description', 'bt']

SCANNER_DESCRIPTION = 'Found by Nemesida Scanner'

TMP_REPORT_FOLDER = Path(tempfile.gettempdir(), 'nwaf-cabinet/reports')
if os.getenv('DEV'):
    TMP_REPORT_FOLDER = Path(Path.cwd(), 'tmp/report/')

if not TMP_REPORT_FOLDER.exists():
    TMP_REPORT_FOLDER.mkdir(parents=True)

FONT_PDF = 'Calibril'
FONT_TABLE_PDF = 'FreeSans'
TABLE_FONT_SIZE = 9
PDF_TABLE_DIVIDER = 2000  # affects on pdf generation speed
PDF_FILE_DIVIDER = 100_000  # affects on pdf generation speed
MIN_ATTACK_TO_SEND_REPORT_TO_MAIL = 10000
