import os

from django.conf import settings
from django.urls import re_path, include, path

import user.login
import user.user_settings
from main import views
from django.contrib.auth import views as auth_views
from django.views.i18n import JavaScriptCatalog
from django.contrib.auth.decorators import login_required

from main.views import tab_EC_management, EC_management

home_page = settings.HOME_PAGE[:-1]

app_name = "cabinet"

urlpatterns = [
    # user urls
    re_path(r'^set_lang/(?P<lang>.+)/', user.user_settings.set_lang, name='set_language'),
    re_path(r'^two_factor_auth/$', user.login.two_factor_auth, name='2fa'),
    re_path(r'^render_2fa/', views.render_2fa, name='render-2fa'),
    re_path(r'^login/$', user.login.login_view, name='login'),
    re_path(r'^logout/$', user.login.logout_view, name='logout'),
    re_path(r'edit_user_password/', views.edit_user_password, name='Edit user password'),
    re_path(r'user_settings/', views.user_settings, name='User_config'),
    re_path(r'^check_and_turn_on_2fa/$', views.check_and_turn_on_2fa, name='check_and_turn_on_2fa'),

    re_path(r'^admin_panel/', include('admin_panel.urls')),
    re_path(r'^waf_settings/', include('my_waf.urls')),
    re_path(r'^$', views.logs_html, name='Logs'),
    re_path(r'^logs/', views.scroll_attacks, name='Attack_scroll'),
    re_path(r'^set_ip_bl/', views.set_ip_blocked, name='Set ip for block'),
    re_path(r'^wl-request/$', views.wl_request, name='Request_for_whitelist'),
    re_path(r'^scan-request/$', views.scan_request, name='Request_for_scan'),
    re_path(r'^exclude-request/', views.scanner_request, name='Scanner_request'),
    path(r'report/', include('report.urls')),
    # re_path(r'^scanner/settings/', views.scanner_settings, name='scanner-settings'),
    re_path(r'^scanner/compare/', views.scanner_compare_html, name='Scanner Compare'),
    re_path(r'^scanner/', views.scanner_html, name='Scanner'),
    re_path(r'^del_target_settings/', views.del_target_settings, name='Delete_target_settings'),
    re_path(r'^run_scanning/', views.run_scanning, name='Run_scanning'),
    re_path(r'^set_target_settings/', views.set_target_settings, name='Set_target_settings'),
    re_path(r'^get_target_list/', views.get_target_list, name='Get_target_list'),
    re_path(r'^get_target_settings/', views.get_target_settings, name='Get_target_settings'),
    re_path(r'^get_status_scanner/', views.get_status_scanner, name='Get_status_scanner'),
    re_path(r'^get_rule/', views.get_rule, name='Get_rule'),
    re_path(r'^get_long_desc/', views.get_long_desc, name='get long description'),
    re_path(r'^get_large_body/', views.get_large_body, name='get large body'),
    re_path(r'^get_rule_wl/', views.get_rule_wl, name='Get rule WL'),
    re_path(r'^add_wl/', views.add_wl, name='Add WL'),
    re_path(r'^get_recheck/', views.get_recheck_status, name='Get_recheck'),
    re_path(r'^charts/', views.charts, name='Charts'),
    re_path(r'^second_row/', views.second_row, name='Second_row'),
    re_path(r'^get_logs_group/', views.get_logs_group, name='Get_logs_group'),
    re_path(r'^get_group_tags/', views.get_group_tags, name='Get_group_tags'),
    re_path(r'^get_country_tags/', views.get_country_tags,
            name='Get_country_tags'),
    re_path(r'^get_scan_chart/', views.get_scan_chart, name='Get_scan_chart'),
    re_path(r'^get_scan_logs/', views.get_scan_logs, name='Get_scan_logs'),
    re_path(r'^ip_info/', views.ip_info, name='Ip_info'),
    re_path(r'^last_filter/$', views.get_last_filter, name='Get_last_filter'),
    re_path(r'^list_filters/$', views.get_list_filters, name='Get_list_filter'),
    re_path(r'^last_scan/$', views.get_last_scan, name='Get_last_scan'),
    re_path(r'^edit_notification_config/',
            login_required(views.NotificationConfigView.as_view()), name='Notification Config'),
    re_path(r'^edit_domain_config/', login_required(views.DomainsConfigView.as_view()),
            name='Change User Domains'),
    re_path(r'openapi/paths/', views.get_openapi_paths, name='OpenAPI paths'),
    re_path(r'openapi/exclude-paths/', views.get_openapi_exclude_paths,
            name='OpenAPI exclude paths'),
    re_path(r'openapi/', views.open_api_settings, name='OpenAPI'),
    re_path(r'load_openapi_schema/', views.load_openapi_schema,
            name='Load OpenAPI schema'),
    re_path(r'delete_openapi_schema/', views.delete_openapi_schema,
            name='Delete OpenAPI schema'),
    re_path(r'delete_from_openapi_schema/',
            views.delete_url_from_openapi_schema, name='Delete path OpenAPI schema'),
    re_path(r'get_openapi_paths/', views.get_openapi_paths,
            name='Get OpenAPI paths'),
    re_path(r'delete_openapi_path/', views.delete_openapi_path,
            name='Delete OpenAPI paths'),
    re_path(r'merge_schema/', views.merge_schema,
            name='Merge OpenAPI schemas'),
    re_path(r'exclude_openapi_path/', views.exclude_openapi_path,
            name='Exclude OpenAPI path'),
    re_path(r'remove_exclude_openapi_paths/', views.remove_exclude_openapi_path, name='Remove exclude OpenAPI path'),
    re_path(r'download_open_api_schema/', views.download_open_api_schema, name='Download OpenAPI schema'),
    re_path(r'remove_schema_path/', views.remove_schema_path, name='Remove OpenAPI schema path'),
    re_path(r'^password_reset/$', views.password_reset_request, name='password_reset'),
    re_path(r'^password_reset/done/$', auth_views.PasswordResetDoneView.as_view(), name='password_reset_done'),
    re_path(r'^reset/(?P<uidb64>[0-9A-Za-z]+)/(?P<token>.+)/$', views.password_reset_confirm,
            name='password_reset_confirm'),
    re_path(r'^reset/done/$', auth_views.PasswordResetCompleteView.as_view(extra_context={'home': home_page}),
            name='password_reset_complete'),
    re_path(r'^model_tuning/', include('model_tuning.urls')),
    re_path(r'^ec/', tab_EC_management),
    re_path(r'^ec_man/', EC_management),
    # re_path(r'^nginx/', include('nginx.urls')),

]

urlpatterns += [
    re_path(r'^jsi18n/', JavaScriptCatalog.as_view(),
            name='javascript-catalog'),
]

if os.getenv('DEV'):
    # translate package
    # install django-rosetta
    urlpatterns += re_path(r'^rosetta/', include('rosetta.urls')),
    urlpatterns += path('__debug__/', include('debug_toolbar.urls')),
