def convert_definition_path(item):
    if '$ref' in item:
        item['$ref'] = item['$ref'].replace('/definitions/', '/components/schemas/')


def combine_description(item):
    if 'description' in item:
        item['description'] = item['description'].replace('\n', ' ')


def convert_property(prop):
    convert_definition_path(prop)
    if 'x-nullable' in prop:
        prop['nullable'] = prop['x-nullable']
        del (prop['x-nullable'])
    if 'items' in prop:
        convert_definition_path(prop['items'])
    if 'properties' in prop:
        for sub_prop in prop['properties'].values():
            convert_property(sub_prop)


def convert_definition(path, def_ob):
    if 'type' in def_ob or '$ref' in def_ob:
        convert_property(def_ob)
    if 'properties' in def_ob:
        for prop_name, prop_ob in def_ob['properties'].items():
            convert_property(prop_ob)
    for comb in ('allOf', 'oneOf', 'anyOf', 'not'):
        for sub_def in def_ob.get(comb, []):
            convert_definition(path + '.' + comb, sub_def)
    if 'items' in def_ob:
        convert_definition(path + '.items', def_ob['items'])


def convert_parameter(param):
    convert_definition_path(param)
    combine_description(param)
    if 'schema' in param:
        convert_definition_path(param['schema'])
        if 'items' in param['schema']:
            convert_definition_path(param['schema']['items'])
    if 'type' in param and param.get('in') != 'formData':
        if 'schema' not in param:
            param['schema'] = {}
        if param.get('type') in ('array', 'boolean', 'integer', 'number', 'object', 'string'):
            param['schema']['type'] = param.get('type')
        elif param.get('type') == 'file':
            param['schema']['type'] = 'string'
            param['schema']['format'] = 'binary'
        else:
            param['schema']['type'] = param.get('type')
        del (param['type'])
        fields_for_schema = ('format', 'pattern', 'default', 'enum', 'minimum', 'maximum', 'exclusiveMinimum',
                             'exclusiveMaximum', 'multipleOf', 'minLength', 'maxLength', 'maxItems', 'minItems',
                             'uniqueItems', 'minProperties', 'maxProperties', 'nullable')
        for field in fields_for_schema:
            if field in param:
                param['schema'][field] = param[field]
                del (param[field])
        if 'collectionFormat' in param:
            del (param['collectionFormat'])
        if 'style' in param:
            del (param['style'])
        if 'items' in param:
            param['schema']['items'] = param['items']
            del (param['items'])
            convert_definition_path(param['schema']['items'])
        if 'aliases' in param:
            del (param['aliases'])
    if 'in' in param and param['in'] not in ('query', 'header', 'path', 'cookie'):
        if param['in'] == 'body':
            pass
        elif param['in'] != 'formData':

            param['in'] = 'query'


def convert_parameters(item):
    if 'parameters' in item:
        removed_items = []
        for param in item['parameters']:
            convert_parameter(param)
            if 'in' in param and param['in'] in ('body', 'formData'):
                if type(item.get('consumes', ['*/*'])) != list:
                    consumes = [item.get('consumes', ['*/*']), ]
                else:
                    consumes = item.get('consumes', ['*/*'])
                if 'requestBody' not in item:
                    item['requestBody'] = {
                        'content': {
                            consumes[0]: {
                                'schema': dict()
                            }
                        }
                    }
                if param['in'] == 'formData':
                    content_type = []
                    if 'content' not in item['requestBody']:
                        item['requestBody']['content'] = dict()

                    tmp = {param['name']: dict()}
                    if 'type' in param:
                        if param.get('type') in ('array', 'boolean', 'integer', 'number', 'object', 'string'):
                            tmp[param['name']]['type'] = param.get('type')
                        elif param.get('type') == 'file':
                            tmp[param['name']]['type'] = 'string'
                            tmp[param['name']]['format'] = 'binary'
                        else:
                            continue

                    if 'description' in param:
                        tmp[param['name']]['description'] = param.get('description')
                    if 'schema' in param:
                        tmp[param['name']]['schema'] = param.get('schema')
                    if 'required' in param and param.get('required'):
                        tmp[param['name']]['required'] = param.get('required')
                    if 'application/x-www-form-urlencoded' in consumes:
                        content_type.append('application/x-www-form-urlencoded')
                    if 'multipart/form-data' in consumes:
                        content_type.append('multipart/form-data')
                    if not content_type:
                        content_type.append('*/*')
                    for c_type in content_type:
                        if c_type not in item['requestBody']['content']:
                            item['requestBody']['content'][c_type] = {'schema': dict()}
                        if 'schema' not in item['requestBody']['content'][c_type]:
                            item['requestBody']['content'][c_type] = {'schema': dict()}
                        if not item['requestBody']['content'][c_type]['schema'].get('properties'):
                            item['requestBody']['content'][c_type]['schema']['properties'] = dict()
                        item['requestBody']['content'][c_type]['schema']['properties'].update(tmp)
                elif param['in'] == 'body':
                    for c_type in consumes:
                        if c_type not in item['requestBody']['content']:
                            item['requestBody']['content'][c_type] = {'schema': dict()}
                        if not item['requestBody']['content'][c_type].get('schema'):
                            item['requestBody']['content'][c_type] = {'schema': dict()}
                        item['requestBody']['content'][c_type]['schema'] = param['schema']
                # Remove the parameter from the item's parameter list
                removed_items.append(param)
        for i in removed_items:
            item['parameters'].remove(i)


def convert_responses(item, produces_list):
    if 'responses' in item:
        for resp_code, response in item['responses'].items():
            if 'schema' not in response and 'examples' not in response:
                continue
            elif 'schema' not in response and 'examples' in response:
                response['schema'] = {'example': response['examples']}
                del response['examples']
            if (
                    'schema' in response and  # noqa
                    'properties' in response['schema']  # noqa
            ):
                for prop_name, prop_data in response['schema']['properties'].items():
                    convert_property(prop_data)
            elif 'schema' in response:
                convert_definition_path(response['schema'])
                if 'items' in response['schema']:
                    convert_definition_path(response['schema']['items'])
            response['content'] = {
                produces: {
                    'schema': response['schema'],
                }
                for produces in produces_list
            }
            if 'headers' in response:
                for header, head_ob in response['headers'].items():
                    convert_parameter(head_ob)
            del (response['schema'])


def convert_schema(api):
    # Make changes

    # Update version
    if 'swagger' in api:
        del (api['swagger'])
    if 'openapi' not in api:
        api['openapi'] = '3.0.0'

    # Convert host to server
    scheme = 'https'
    base_path = api.get('basePath', '/')
    description = api['info'].get('description', 'No server description given')
    if 'host' in api:
        host_list = api['host'] if isinstance(api['host'], list) else [api['host']]
        del (api['host'])

        hosts = [
            {
                'url': f"{scheme}://{host}{base_path}",
                'description': description,
            }
            for host in host_list
        ]
        del (api['basePath'])
        api['servers'] = hosts

    # Convert path information

    produces = api.get('produces', ['application/json'])

    for path, path_ob in api['paths'].items():
        convert_parameters(path_ob)
        for op_name, op_ob in path_ob.items():
            op_produces = produces
            if 'produces' in op_ob:
                op_produces = op_ob.pop('produces')
            convert_parameters(op_ob)
            convert_responses(op_ob, op_produces)
            combine_description(op_ob)
            if 'consumes' in op_ob:
                del (op_ob['consumes'])

    # Convert definitions into components

    for def_name, def_ob in api.get('definitions', dict()).items():
        convert_definition(def_name, def_ob)

    api['components'] = {
        'schemas': api.get('definitions', dict())
    }
    if 'definitions' in api:
        del (api['definitions'])

    # Unneeded keys:

    for key in ('consumes', 'produces', 'schemes', 'securityDefinitions'):
        if key in api:
            if key == 'securityDefinitions':
                sec_def = dict()
                for sec_key in api.get(key).keys():
                    sec_def.update({sec_key: dict()})
                    sec_par = api[key][sec_key]
                    if 'flow' in sec_par:
                        if type(sec_par['flow']) != list:
                            sec_par['flow'] = [sec_par['flow'], ]
                        for flow in sec_par['flow']:
                            if 'flows' not in sec_par:
                                sec_par['flows'] = dict()
                            sec_par['flows'][flow] = dict()
                            for i in ('authorizationUrl', 'tokenUrl', 'refreshUrl', 'scopes'):
                                if i in sec_par:
                                    sec_par['flows'][flow][i] = sec_par[i]
                        for i in ('authorizationUrl', 'tokenUrl', 'refreshUrl', 'scopes', 'flow'):
                            if i in sec_par:
                                del sec_par[i]
                        sec_def[sec_key].update(sec_par)
                    else:
                        sec_def[sec_key].update(sec_par)
                if 'securitySchemes' not in api['components']:
                    api['components']['securitySchemes'] = dict()
                api['components']['securitySchemes'].update(sec_def)

            del (api[key])

    return api
