import time
import logging

from datetime import datetime, timedelta
from django.conf import settings

DEFAULT_DATE_FILTER = settings.DEFAULT_DATE_FILTER
logger = logging.getLogger(__name__)


def start_end_time_decorator(func=None, **kwargs):
    def decorated(func):

        def wrapper(view, request, *args, **kwargs):
            time_clear_date = request.session.get('time_clear_date', datetime.today())
            if type(time_clear_date) == str:
                time_clear_date = datetime.strptime(time_clear_date, "%Y-%m-%d %H:%M:%S.%f")
            if time_clear_date <= datetime.today():
                if 'end_date' in request.session.keys():
                    request.session.pop('end_date')
                if 'start_date' in request.session.keys():
                    request.session.pop('start_date')
                request.session.modified = True

            if 'end' in request.GET.keys():
                end_date = datetime.strftime(
                    datetime.strptime(request.GET.get('end'), "%d.%m.%Y").replace(hour=23, minute=59, second=59),
                    "%Y-%m-%d %H:%M:%S.%f"
                )
                if 'end_date' not in request.session.keys() or request.session.get('end_date') != end_date:
                    request.session['end_date'] = end_date
                    request.session.modified = True

            else:
                if 'end_date' in request.session.keys():
                    end_date = request.session['end_date']
                    request.session.modified = True
                else:
                    end_date = datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S.%f")

            if 'start' in request.GET.keys():
                start_date = datetime.strftime(
                    datetime.strptime(request.GET.get('start'), "%d.%m.%Y").replace(hour=00, minute=00, second=00),
                    "%Y-%m-%d %H:%M:%S.%f"
                )

                if 'start_date' not in request.session.keys() or request.session.get('start_date') != start_date:
                    request.session['start_date'] = start_date
                    request.session.modified = True
            else:
                if 'start_date' in request.session.keys():
                    start_date = request.session['start_date']
                    request.session.modified = True
                else:
                    start_date = datetime.strftime(
                        datetime.now() + timedelta(days=-DEFAULT_DATE_FILTER),
                        "%Y-%m-%d %H:%M:%S.%f"
                    )

            if time_clear_date != datetime.today() and \
                    ('start_date' in request.session.keys() or 'end_date' in request.session.keys()):
                tomorrow = datetime.today() + timedelta(days=1)
                request.session['time_clear_date'] = str(tomorrow.replace(hour=00, minute=00, second=00))
                request.session.modified = True

            return func(view, request, start_date, end_date, *args, **kwargs)

        return wrapper

    return decorated(func)


def start_end_time(func=None, **kwargs):
    def decorated(func):

        def wrapper(request, *args, **kwargs):
            time_clear_date = request.session.get('time_clear_date', datetime.today())
            if type(time_clear_date) == str:
                time_clear_date = datetime.strptime(time_clear_date, "%Y-%m-%d %H:%M:%S.%f")
            if time_clear_date <= datetime.today():
                if 'end_date' in request.session.keys():
                    request.session.pop('end_date')
                if 'start_date' in request.session.keys():
                    request.session.pop('start_date')
                request.session.modified = True

            if 'end' in request.GET.keys():
                end_date = datetime.strftime(datetime.strptime(request.GET.get('end'), "%d.%m.%Y")
                                             .replace(hour=23, minute=59, second=59), "%Y-%m-%d %H:%M:%S.%f")
                if 'end_date' not in request.session.keys() or request.session.get('end_date') != end_date:
                    request.session['end_date'] = end_date
                    request.session.modified = True
            else:
                if 'end_date' in request.session.keys():
                    end_date = request.session['end_date']
                    request.session.modified = True
                else:
                    end_date = datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S.%f")

            if 'start' in request.GET.keys():
                start_date = datetime.strftime(datetime.strptime(request.GET.get('start'), "%d.%m.%Y")
                                               .replace(hour=00, minute=00, second=00), "%Y-%m-%d %H:%M:%S.%f")
                if 'start_date' not in request.session.keys() or request.session.get('start_date') != start_date:
                    request.session['start_date'] = start_date
                    request.session.modified = True
            else:
                if 'start_date' in request.session.keys():
                    start_date = request.session['start_date']
                    request.session.modified = True
                else:
                    start_date = datetime.strftime(datetime.now() + timedelta(days=-DEFAULT_DATE_FILTER),
                                                   "%Y-%m-%d %H:%M:%S.%f")

            if time_clear_date != datetime.today() and (
                    'start_date' in request.session.keys() or 'end_date' in request.session.keys()):
                tomorrow = datetime.today() + timedelta(days=1)
                request.session['time_clear_date'] = str(tomorrow.replace(hour=00, minute=00, second=00))
                request.session.modified = True

            return func(request, start_date, end_date, *args, **kwargs)

        return wrapper

    return decorated(func)


def start_end_time_var(func=None, **kwargs):
    def decorated(func):

        def wrapper(request, *args, **kwargs):
            time_clear_date = request.session.get('time_clear_date', datetime.today())
            if type(time_clear_date) == str:
                time_clear_date = datetime.strptime(time_clear_date, "%Y-%m-%d %H:%M:%S.%f")
            if time_clear_date <= datetime.today():
                if 'end_date' in request.session.keys():
                    request.session.pop('end_date')
                if 'start_date' in request.session.keys():
                    request.session.pop('start_date')
                request.session.modified = True

            if 'end' in request.GET.keys():
                end_date = datetime.strftime(datetime.strptime(request.GET.get('end'), "%d.%m.%Y")
                                             .replace(hour=23, minute=59, second=59), "%Y-%m-%d %H:%M:%S.%f")
                if 'end_date' not in request.session.keys() or request.session.get('end_date') != end_date:
                    request.session['end_date'] = end_date
                    request.session.modified = True
            else:
                if 'end_date' in request.session.keys():
                    end_date = request.session['end_date']
                    request.session.modified = True
                else:
                    end_date = datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S.%f")

            if 'start' in request.GET.keys():
                start_date = datetime.strftime(datetime.strptime(request.GET.get('start'), "%d.%m.%Y")
                                               .replace(hour=00, minute=00, second=00), "%Y-%m-%d %H:%M:%S.%f")
                if 'start_date' not in request.session.keys() or request.session.get('start_date') != start_date:
                    request.session['start_date'] = start_date
                    request.session.modified = True
            else:
                if 'start_date' in request.session.keys():
                    start_date = request.session['start_date']
                    request.session.modified = True
                else:
                    start_date = datetime.strftime(datetime.now() + timedelta(days=-DEFAULT_DATE_FILTER),
                                                   "%Y-%m-%d %H:%M:%S.%f")

            if time_clear_date != datetime.today() and (
                    'start_date' in request.session.keys() or 'end_date' in request.session.keys()):
                tomorrow = datetime.today() + timedelta(days=1)
                request.session['time_clear_date'] = str(tomorrow.replace(hour=00, minute=00, second=00))
                request.session.modified = True

            return func(request, start_date, end_date)

        return wrapper

    return decorated(func)


def start_end_time_scanner(func=None, **kwargs):
    def decorated(func):

        def wrapper(request, *args, **kwargs):
            time_clear_date = request.session.get('time_clear_date', datetime.today())
            if type(time_clear_date) == str:
                time_clear_date = datetime.strptime(time_clear_date, "%Y-%m-%d %H:%M:%S.%f")
            if time_clear_date <= datetime.today():
                if 'end_date' in request.session.keys():
                    request.session.pop('end_date')
                if 'start_date' in request.session.keys():
                    request.session.pop('start_date')
                request.session.modified = True

            if 'end' in request.GET.keys():

                end_date = datetime.strftime(
                    datetime.strptime(request.GET.get('end'), "%d.%m.%Y")
                    .replace(hour=23, minute=59, second=59), "%Y-%m-%d %H:%M:%S.%f")
                if 'end_date' not in request.session.keys() or request.session.get('end_date') != end_date:
                    request.session['end_date'] = end_date
                    request.session.modified = True
            else:
                if 'end_date' in request.session.keys():
                    end_date = request.session['end_date']
                    request.session.modified = True
                else:
                    end_date = datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S.%f")

            if 'start' in request.GET.keys():

                start_date = datetime.strftime(
                    datetime.strptime(request.GET.get('start'), "%d.%m.%Y")
                    .replace(hour=00, minute=00, second=00), "%Y-%m-%d %H:%M:%S.%f")
                if 'start_date' not in request.session.keys() or request.session.get('start_date') != start_date:
                    request.session['start_date'] = start_date
                    request.session.modified = True
            else:
                if 'start_date' in request.session.keys():
                    start_date = request.session['start_date']
                    request.session.modified = True
                else:
                    start_date = datetime.strftime(datetime.now() + timedelta(days=-DEFAULT_DATE_FILTER),
                                                   "%Y-%m-%d %H:%M:%S.%f")

            if time_clear_date != datetime.today() and (
                    'start_date' in request.session.keys() or 'end_date' in request.session.keys()):
                tomorrow = datetime.today() + timedelta(days=1)
                request.session['time_clear_date'] = str(tomorrow.replace(hour=00, minute=00, second=00))
                request.session.modified = True

            return func(request, start_date, end_date, *args, **kwargs)

        return wrapper

    return decorated(func)


def time_of_function(function):
    def wrapped(*args, **kwargs):
        if settings.DEBUG:
            logger.log(
                level=logging.DEBUG,
                msg='Function <{name_func}> running'.format(name_func=function.__name__)
            )

        start_time = time.time()
        res = function(*args, **kwargs)
        delta_time = time.time() - start_time
        if settings.DEBUG or delta_time > settings.LOG_FOR_TIMEOUT:
            if settings.DEBUG:
                lvl = logging.DEBUG
            elif delta_time > settings.LOG_FOR_TIMEOUT:
                lvl = logging.WARNING
            else:
                lvl = logging.INFO

            logger.log(
                level=lvl,
                msg='Function <{name_func}> ended with time: {delta_time}s'
                .format(name_func=function.__name__, delta_time=round(delta_time))
            )

        return res

    return wrapped
