from main.common import get_location_by_ip, get_reason_by_model
from pymemcache.client.base import Client
from django.conf import settings
from urllib.parse import unquote

from utils.mail import send_mail_from_cabinet

cache_ip, cache_port = settings.CACHES['default']['LOCATION'].split(':')
cache_address = (cache_ip, int(cache_port))


def send_email_after_create_whitelist_request(whitelist_request):
    subject = "Nemesida WAF: unlock request"
    mcache = Client(cache_address)
    location = get_location_by_ip(mcache, whitelist_request.log.ip)
    reason = get_reason_by_model(whitelist_request.log.bt)
    
    mz = whitelist_request.log.mz
    mz = 'Path' if mz in ['URL', 'URI'] else mz

    body = "{0} {1} in {2}\n\n{3:12} {4}\n{5:12} {6}\n{7:12} {8}\n{9:12} {10}\n{11:12} {12}\n\n" \
           "{13:12} {14} {15}{16}\n{17:12} {18}\n{19:12} {20}\n{21:12} {22}\n\n{23:12} {24}".format(
            "Blocked by", reason, mz,
            "Host:", whitelist_request.log.vhost,
            "Path:", whitelist_request.log.url,
            "ARGS:", whitelist_request.log.param,
            "UA:", whitelist_request.log.ua,
            "Cookie:", whitelist_request.log.cookie,
            "IP:", whitelist_request.log.ip,
            '' if location.get('city') in (None, "None") else location.get('city'),
            '' if location.get('country') in (None, "None") else location.get('country'),
            "Date:", whitelist_request.log.timestamp.strftime("%H:%M:%S %d.%m.%Y"),
            "Request ID:", whitelist_request.log.request_id,
            "WAF ID:", whitelist_request.log.alias,
            "Comment:", whitelist_request.user_comment,
            )

    body = unquote(unquote(body))
    send_mail_from_cabinet(whitelist_request.user.email, subject, body)


def send_email_after_done_whitelist_request(whitelist_request):
    subject = "Nemesida WAF: request successfully processed"
    mcache = Client(cache_address)
    location = get_location_by_ip(mcache, whitelist_request.log.ip)

    body = "{0:12} {1}\n{2:12} {3}\n{4:12} {5}\n{6:12} {7}\n{8:12} {9}\n\n{10:12} {11} {12}{13}\n{14:12} {15}\n" \
           "{16:12} {17}\n{18:12} {19}\n\n{20:12} {21}".format(
            "Host:", whitelist_request.log.vhost,
            "Path:", whitelist_request.log.url,
            "ARGS:", whitelist_request.log.param,
            "UA:", whitelist_request.log.ua,
            "Cookies", whitelist_request.log.cookie,
            "IP:", whitelist_request.log.ip,
            '' if location.get('city') in (None, "None") else location.get('city'),
            '' if location.get('country') in (None, "None") else location.get('country'),
            "Date:", whitelist_request.log.timestamp.strftime("%H:%M:%S %d.%m.%Y"),
            "Request ID:", whitelist_request.log.request_id,
            "WAF ID:", whitelist_request.log.alias,
            "Comment:", whitelist_request.admin_comment,
            )

    body = unquote(unquote(body))
    send_mail_from_cabinet(whitelist_request.user.email, subject, body)
