import struct
import os
import pcre
import urllib.parse
from main.fields import FieldNames
from main.common import parse_body

# templ = '<BIII{}sI{}sII{}s'.format(pt_len, sc_name_len, zone_len)

MZ_ZONES = dict()

MZ_BODY = 1 << 0
MZ_URL = 1 << 1
MZ_ARGS = 1 << 2
MZ_HEADERS = 1 << 3
MZ_FILE_EXT = 1 << 4
MZ_FILE_DATA = 1 << 5
MZ_SPLIT_EVSN = 1 << 6

IS_WL = 1
IS_RX = 1 << 1
IS_LM = 1 << 2


class Rule(object):
    mz_flags = 0
    flags = 0
    id = None
    target = None
    score_k = None
    score_v = None
    url_k = None

    def __init__(self):
        self.keys = []


class NWAF_emu(object):
    def __init__(self, file):
        self.rules = []
        self.rule_parser(file)

    @staticmethod
    def zone_parse(rule, s):
        for i in s.split('|'):
            if i not in MZ_ZONES.keys():
                MZ_ZONES[i] = 1 << len(MZ_ZONES)
            if i in MZ_ZONES:
                rule.mz_flags |= MZ_ZONES[i]
            elif i == 'FILE_EXT':
                rule.mz_flags |= MZ_FILE_EXT
            elif i == 'FILE_DATA':
                rule.mz_flags |= MZ_FILE_DATA
            elif i == 'SPLIT_EVASION':
                rule.mz_flags |= MZ_SPLIT_EVSN
            elif '$URL:' in i:
                rule.url_k = i.replace('$URL:', '')
            elif '$HEADERS_VAR:' in i:
                rule.keys.append((i.replace('$HEADERS_VAR:', ''), MZ_HEADERS))

    # Creates rules as Rule class objects from the rules.bin file, then
    # adds these objects to the rules list
    def rule_parser(self, file):
        endian = '<'
        static_d = endian + 'BII'
        int_d = endian + 'I'
        int_size = struct.calcsize(int_d)
        str_tpl = endian + '{}s'

        f = open(file, 'rb')
        f.seek(0, os.SEEK_END)
        file_len = f.tell()
        f.seek(0)

        self.rules = []
        while f.tell() < file_len:
            rl = Rule()
            length = struct.calcsize(static_d)
            rnd, rl.id, rl.flags = struct.unpack(static_d, f.read(length))

            str_len, = struct.unpack(int_d, f.read(int_size))
            str_d = str_tpl.format(str_len)
            length = struct.calcsize(str_d)
            pt, = struct.unpack(str_d, f.read(length))
            rl.target = bytes([x ^ rnd for x in pt]).decode('ascii')

            str_len, = struct.unpack(int_d, f.read(int_size))
            str_d = str_tpl.format(str_len)
            length = struct.calcsize(str_d)
            pt, = struct.unpack(str_d, f.read(length))
            rl.score_k = bytes([x ^ rnd for x in pt]).decode('ascii')
            rl.score_v, = struct.unpack(int_d, f.read(int_size))

            str_len, = struct.unpack(int_d, f.read(int_size))
            str_d = str_tpl.format(str_len)
            length = struct.calcsize(str_d)
            pt, = struct.unpack(str_d, f.read(length))
            self.zone_parse(rl, bytes([x ^ rnd for x in pt]).decode('ascii'))
            self.rules.append(rl)
        f.close()

    def get_rule(self, rule_id):
        res = []
        for rule in self.rules:
            if rule.id == int(rule_id):
                target_rule = rule.__dict__
                target_rule["zone"] = []
                for name, zone in MZ_ZONES.items():
                    if rule.mz_flags & zone:
                        target_rule["zone"].append(name)
                res.append(target_rule)
        return res[0] if len(res) > 0 else {}

    @staticmethod
    def get_entry(src, rule):

        def build_entry(src, rule):
            res = dict()
            res[src] = [
                {
                    "id": rule.id,
                    "target": rule.target,
                    "score_k": rule.score_k,
                    "score_v": rule.score_v,
                    "flags": rule.flags
                }
            ]
            return res

        if rule.flags & IS_RX:
            try:
                pattern = pcre.compile(rule.target, pcre.IGNORECASE)
                result = pattern.findall(src.lower())
                if result:
                    return build_entry(src, rule)
            except Exception:
                return None
        else:
            data = src.lower()
            ind = data.find(rule.target.lower())
            if ind > -1:
                return build_entry(src, rule)

    def get_rule_entry(self, val, mz, row, key=None):
        res = {}
        wl_res = {}
        val = urllib.parse.unquote(urllib.parse.unquote(val))
        for rule in self.rules:
            if rule.mz_flags & mz:
                t = self.get_entry(val, rule)
                if t:
                    if rule.url_k:
                        if rule.url_k == row['uri']:
                            if rule.flags & IS_WL:
                                wl_res[val] = res.get(val, list()) + t[val]
                            else:
                                res[val] = res.get(val, list()) + t[val]
                    else:
                        if rule.flags & IS_WL:
                            wl_res[val] = res.get(val, list()) + t[val]
                        else:
                            res[val] = res.get(val, list()) + t[val]
            if key is None:
                continue
            for k, z in rule.keys:
                if not (z & mz) or key.lower() != k.lower():
                    continue
                t = self.get_entry(val, rule)
                if t:
                    if rule.flags & IS_WL:
                        wl_res[val] = res.get(val, list()) + t[val]
                    else:
                        res[val] = res.get(val, list()) + t[val]
        # wl
        for i in res:
            for num, v in enumerate(res[i]):
                if v["flags"] & IS_RX:
                    pattern = pcre.compile(v["target"], pcre.IGNORECASE)
                    try:
                        substring = pattern.findall(val.lower())[0]
                    except Exception:
                        substring = v["target"]
                else:
                    substring = v["target"]
                for r in wl_res:
                    for vl in wl_res[r]:
                        if vl["flags"] == 3:
                            pattern = pcre.compile(vl["target"], pcre.IGNORECASE)
                            result = pattern.findall(substring.lower())
                            if result:
                                res[i][num] = {}
                        elif vl["flags"] == 1:
                            data = vl["target"].lower()
                            ind = data.find(substring.lower())
                            if ind > -1:
                                res[i][num] = {}
        for i in res:
            res[i] = [v for v in res[i] if v != {}]
        res = {k: v for k, v in res.items() if v != []}
        return res

    def custom_request(self, request):
        res = {}
        field_names = FieldNames().__dict__
        t = self.get_rule_entry(request['uri'], MZ_ZONES.get('URL', MZ_URL), request, 'URL')
        if t:
            res[field_names['uri']] = t
        t = self.get_rule_entry(request['args'], MZ_ZONES.get('ARGS', MZ_ARGS), request)
        if t:
            res[field_names['args']] = t
        t = self.get_rule_entry(request['cookie'], MZ_ZONES.get('Cookie', MZ_HEADERS), request, 'Cookie')
        if t:
            res[field_names['cookie']] = t
        t = self.get_rule_entry(request['ua'], MZ_ZONES.get('User-Agent', MZ_HEADERS), request, 'User-Agent')
        if t:
            res[field_names['ua']] = t
        t = self.get_rule_entry(request['referer'], MZ_ZONES.get('Referer', MZ_HEADERS), request, 'Referer')
        if t:
            res[field_names['referer']] = t
        try:
            t = self.get_rule_entry(request['content_t'], MZ_ZONES.get('Content-Type', MZ_HEADERS),
                                    request, 'Content-Type')
            if t:
                res[field_names['content_t']] = t
        except KeyError:
            pass
        t = self.get_rule_entry(parse_body(request['body']), MZ_ZONES.get('BODY', MZ_BODY), request)
        if t:
            res[field_names['body']] = t
        for i in request["other_headers"].splitlines():
            if len(i.split(':')) < 2:
                continue
            t = self.get_rule_entry(i.split(':')[1], MZ_ZONES.get('HEADERS', MZ_HEADERS), request, i.split(':')[0])
            if t:
                if i.split(':')[0] == 'content-type':
                    res[field_names['content_t'] + ' '] = t
                else:
                    res[field_names['other_headers']] = t
        return res
