import os
import sys
import asyncio
import logging
import requests
import datetime
import psycopg2
import psycopg2.extras
from django.conf import settings
from url_normalize import url_normalize

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.append(BASE_DIR)

# Do not change the import
from main.utils import get_proxy_api_dict


BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.append(BASE_DIR)

if os.environ.get('DJANGO_SETTINGS_MODULE') is None:
    os.environ['DJANGO_SETTINGS_MODULE'] = "cabinet.settings_extra"

try:
    api_url = settings.API_URI
except AttributeError:
    api_url = 'http://localhost:8080/nw-api/'

# logging
log_place = '/var/log/uwsgi/cabinet/cleaning_db.log'

# Key alive time
debug = False

# Postgres connect
DB_NAME = settings.DB_NAME_CONF
DB_USER = settings.DB_USER_CONF
DB_PASS = settings.DB_PASS_CONF
DB_HOST = settings.DB_HOST_CONF

try:
    DB_PORT = settings.DB_PORT_CONF
    if DB_PORT == '':
        DB_PORT = '5432'
except NameError:
    DB_PORT = '5432'

try:
    DB_PORT_CABINET = settings.DB_PORT_CABINET
    if DB_PORT_CABINET == '':
        DB_PORT_CABINET = '5432'
except NameError:
    DB_PORT_CABINET = '5432'


user_agent = {'User-agent': settings.USER_AGENT}
result = []


def clear_attacks(loop):
    try:
        connect_cabinet = psycopg2.connect(
            database=settings.DB_NAME_CABINET,
            user=settings.DB_USER_CABINET,
            password=settings.DB_PASS_CABINET,
            host=settings.DB_HOST_CABINET,
            port=DB_PORT_CABINET,
            connect_timeout=10
        )
        connect_cabinet.set_session(autocommit=True)

        cursor_cabinet = connect_cabinet.cursor(cursor_factory=psycopg2.extras.DictCursor)
        cursor_cabinet.execute("SELECT * FROM config_cleaning_attack where enable = true;")
        tasks = cursor_cabinet.fetchall()
        cursor_cabinet.close()
        connect_cabinet.close()

        connect_waf = psycopg2.connect(database=DB_NAME, user=DB_USER, password=DB_PASS, host=DB_HOST, port=DB_PORT, connect_timeout=10)
        connect_waf.set_session(autocommit=True)
        cursor_waf = connect_waf.cursor()

        for task in tasks:
            if debug:
                sql = "SELECT ip, vhost, timestamp, url, bt FROM attack WHERE {time_interval} {vhost} {ip} {url} {bt};"
            else:
                sql = "DELETE FROM attack WHERE {time_interval} {vhost} {ip} {url} {bt};"
            time_interval = ''
            vhost = ''
            url = ''
            ip = ''
            bt = ''
            if task.get('time_interval') not in (0, None, ''):
                time_interval = "(current_date - timestamp)  > INTERVAL '{} day' AND".format(task.get('time_interval'))
            if task.get('vhost').strip(' ') not in ('*', None, ''):
                vhost = task.get('vhost')
                if ',' in vhost:
                    vh_str = ''
                    for i in vhost.replace(" ", '').split(','):
                        if i[0] in ('*', '.'):
                            vh_str += "vhost ILIKE '%{}'".format(i[1:])
                        else:
                            vh_str += "vhost = '{}'".format(i)

                        if vhost.replace(" ", '').split(',')[-1] != i:
                            vh_str += ' or '
                    vhost = '({}) AND'.format(vh_str)
                else:
                    if vhost[0] in ('.', '*'):
                        vhost = "vhost ILIKE '%{}' AND".format(vhost[1:])
                    else:
                        vhost = "vhost = '{}' AND".format(vhost)
            if task.get('ip').strip(' ') not in ('*', None, ''):
                ip = task.get('ip').replace(' ', '')

                if ',' in task.get('ip'):
                    ip_str = ''
                    for i in ip.split(','):
                        ip_str += '\'{}\''.format(i)
                        if i != ip.split(',')[-1]:
                            ip_str += ','
                    ip = "ip in ({}) AND".format(ip_str)
                else:
                    ip = "ip = '{}' AND".format(ip)
            if task.get('url').strip(' ') not in ('*', None, ''):
                url = "url = '{}' AND".format(task.get('url'))
            if task.get('bt').strip(' ') not in ('*', None, ''):
                bt = task.get('bt')
                if ',' in bt:
                    bt = "bt IN ({})".format(bt.replace(' ', ''))
                else:
                    if bt.isdigit():
                        bt = "bt = {}".format(bt)

            ressql = sql.format(time_interval=time_interval, vhost=vhost, ip=ip, url=url, bt=bt)
            if bt == '':
                if ' AND ' in ressql:
                    ressql = '{}{}'.format(ressql[:ressql.rindex(' AND ')],
                                           ressql[ressql.rindex(' AND '):].replace(' AND ', ''))
                elif time_interval == '' and url == '' and ip == '' and vhost == '':
                    ressql = '{}'.format(ressql.replace(' WHERE ', ''))

            cursor_waf.execute(ressql)
            if debug:
                res = cursor_waf.fetchall()
                logging.debug('Task = {}'.format(task))
                logging.debug('Deleting rows = {}'.format(res))
                logging.debug('delete rows = {}'.format(cursor_waf.rowcount))

        cursor_waf.close()
        connect_waf.close()

    except Exception as error:
        logging.error(
            'An error occurred when an unknown error occurred while clearing the attack table: {}'
            .format(error)
        )

    loop.call_later(30, clear_attacks, loop)


def erl_end_of_life(loop):
    now_datetime = datetime.datetime.timestamp(datetime.datetime.now())
    try:
        connect = psycopg2.connect(
            database=settings.DB_NAME_CABINET,
            user=settings.DB_USER_CABINET,
            password=settings.DB_PASS_CABINET,
            host=settings.DB_HOST_CABINET,
            port=DB_PORT_CABINET,
            connect_timeout=10
        )
        connect.set_session(autocommit=True)
        cursor = connect.cursor()
        cursor.execute("SELECT rid,end_timestamp,settings_lickeys.key FROM settings_erl_end_of_life "
                       "INNER JOIN settings_lickeys on key_id=id;")
        for record in cursor.fetchall():
            if now_datetime >= float(record[1]):
                full_settings_dict = {"key": record[2], 'upd': {'rid': int(record[0]), 'active': False}}
                set_response = requests.post(
                    url_normalize(api_url + '/set_dyn_erl'),
                    headers={'Use-agent': settings.USER_AGENT},
                    proxies=get_proxy_api_dict(),
                    timeout=30,
                    json=full_settings_dict,
                )
                if set_response.ok:
                    cursor.execute("DELETE FROM settings_erl_end_of_life WHERE rid=%s", (record[0],))
        cursor.close()
        connect.close()
    except Exception as error:
        logging.error('ERL EOL error: ' + str(error))

    loop.call_later(60, erl_end_of_life, loop)


# Event Loop
def main():
    logging.basicConfig(
        filename=log_place,
        level=logging.DEBUG if debug else logging.INFO,
        format='%(asctime)s %(levelname)s: %(message)s',
        datefmt='%d/%b/%Y:%H:%M:%S %z')
    logging.info('Started')
    loop = asyncio.get_event_loop()
    try:
        loop.call_soon(clear_attacks, loop)
        loop.call_soon(erl_end_of_life, loop)
        loop.run_forever()
    finally:
        loop.close()
        logging.info('Finished')


if __name__ == '__main__':
    main()
