import asyncio
import sys
import requests
import hashlib
import os

from django.conf import settings

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.append(BASE_DIR)

# Do not change the import
from main.utils import get_proxy_conf_dict

if os.environ.get('DJANGO_SETTINGS_MODULE') is None:
    os.environ['DJANGO_SETTINGS_MODULE'] = "cabinet.settings_extra"

headers = {'User-agent': 'Nemesida WAF Cabinet Rules Update Service'}
host = settings.URI_RL


class NWAFGetConfig(object):

    def __init__(self):
        nwaf_rules = str(BASE_DIR) + "/rules/rules.bin"
        if not os.path.isfile(nwaf_rules):

            with open(nwaf_rules, 'w') as f:
                f.close()

        self.nwaf_rules = nwaf_rules
        self.proxy = get_proxy_conf_dict()

        self.hash = {}
        self.limits_hash = ''
        self.rules_hash = ''
        self._reload = False
        self.check_delay = 0
        self.lic_ext = None
        self.last_update = 0

    @staticmethod
    def file_clearning(filename):
        with open(filename, 'w') as ff:
            ff.seek(0)
            ff.truncate()

    def rule_update(self):
        rules = ''
        updateble = False
        s = requests.Session()

        try:

            # get the timestamp
            timestamp_from_server = s.get(host + "/ts", proxies=self.proxy, headers=headers)

            if not timestamp_from_server.ok:
                raise Exception(
                    'Nemesida WAF Rule Update: an error occurred while getting the timestamp, server response code {}'
                    .format(timestamp_from_server.status_code)
                )

            # load as int
            timestamp = int(timestamp_from_server.text)

            # timestamp processing
            if timestamp != self.last_update:
                self.last_update = timestamp
                updateble = True

                # get the ruleset
                rules = s.get(host + "/rs", proxies=self.proxy, headers=headers)

        except Exception as err:
            print('Nemesida WAF Rule Update: {}'.format(err))

        else:

            if updateble:

                m = hashlib.sha512()
                m.update(rules.content)
                new_hash = m.hexdigest()

                with open(self.nwaf_rules, 'rb') as fr:
                    file_content = fr.read()
                n = hashlib.sha512()
                n.update(file_content)
                old_hash = n.hexdigest()

                if new_hash != old_hash:
                    if self.rules_load(rules.content):
                        pass

    def rules_load(self, data=None):
        with open(self.nwaf_rules, 'wb') as rl:
            rl.write(data)


def nwaf_conf_handlers(loop, delay, *args):
    for i in args:
        if hasattr(i, '__call__'):
            i()
    loop.call_later(delay, nwaf_conf_handlers, loop, delay, *args)


def main():
    loop = asyncio.get_event_loop()
    try:
        upd = NWAFGetConfig()
        loop.call_soon(nwaf_conf_handlers, loop, 30, upd.rule_update)

        loop.run_forever()
    except KeyboardInterrupt:
        exit(0)
    finally:
        loop.close()


if __name__ == '__main__':
    main()
