from django.db import models
from django.db.models import DEFERRED
from .model_tuning import TableQuerySet


class Logs(models.Model):
    class Meta:
        db_table = 'attack'

    timestamp = models.DateTimeField(blank=False,
                                     verbose_name='Date')
    vhost = models.TextField(max_length=255,
                             blank=False,
                             verbose_name='Domain')
    ip = models.TextField(blank=False,
                          verbose_name="Attacker's IP")
    rule_id = models.PositiveIntegerField(blank=False,
                                          verbose_name='Rule id')
    method = models.TextField(max_length=1024,
                              blank=False,
                              verbose_name='Method')
    url = models.TextField(max_length=1024,
                           blank=False,
                           verbose_name='URL')
    body = models.BinaryField(max_length=1024,
                              verbose_name='Body')
    lm = models.BooleanField(blank=True,
                             verbose_name='Learning Mode')
    mz = models.TextField(blank=True,
                          verbose_name='Zone')
    param = models.TextField(blank=True,
                             verbose_name='Parameter')
    ua = models.TextField(blank=True,
                          verbose_name='User-agent')
    referer = models.TextField(blank=True,
                               verbose_name='Referer')
    other_headers = models.TextField(blank=True,
                                     verbose_name='Other headers')
    description = models.TextField(blank=True,
                                   verbose_name='Comment')
    bt = models.IntegerField(default=0,
                             verbose_name='Attack Model')
    cookie = models.TextField(blank=True,
                              verbose_name='Cookies',
                              null=True)
    group_id = models.TextField(blank=True,
                                verbose_name='Group ID',
                                null=True)
    blocked = models.BooleanField(blank=True,
                                  verbose_name='Blocked',
                                  null=True)
    request_id = models.TextField(blank=True, verbose_name='Request_id',
                                  null=True)
    alias = models.TextField(blank=True,
                             null=True,
                             db_column='waf_id',
                             verbose_name='WAF ID')
    possible = models.BooleanField(blank=True,
                                   verbose_name='Possible',
                                   null=True)
    cc = models.CharField(max_length=2,
                          default='',
                          blank=True,
                          null=True,
                          verbose_name='Country Code')
    long_description = models.TextField(blank=True,
                                        null=True,
                                        db_column='long_description',
                                        verbose_name='long description')

    objects_for_mt = TableQuerySet.as_manager()
    objects = models.Manager()

    def __str__(self):
        return str(self.id)

    @classmethod
    def from_db(cls, db, field_names, values):
        # Default implementation of from_db() (subject to change and could
        # be replaced with super()).
        if len(values) != len(cls._meta.concrete_fields):
            values = list(values)
            values.reverse()
            values = [
                values.pop() if f.attname in field_names else DEFERRED
                for f in cls._meta.concrete_fields
            ]
        instance = cls(*values)
        instance._state.adding = False
        instance._state.db = db

        # customization to store the original field values on the instance
        instance._loaded_values = dict(zip(field_names, values))

        return instance
