from django.db import models
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager


class WAFUSERMANAGER(BaseUserManager):
    def create_user(self, email, password=None):
        if not email:
            raise ValueError('User must have an E-mail')

        user = self.model(
            email=self.normalize_email(email),
        )

        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password):
        user = self.create_user(email,
                                password=password)
        user.is_admin = True
        user.waf_alias = '*'
        user.domain = '*'
        user.save(using=self._db)
        return user


class WAFUSER(AbstractBaseUser):
    email = models.EmailField(
        verbose_name="Superuser's e-mail",
        max_length=255,
        unique=True,
    )
    secret = models.CharField(
        default='0',
        max_length=100,
        verbose_name='Secret code'
    )
    is_2fa_active = models.BooleanField(
        default=False,
        verbose_name='Is 2fa active'
    )
    is_admin = models.BooleanField(
        default=False,
        verbose_name='Administrator'
    )
    priv_readuser = models.BooleanField(
        default=True,
        verbose_name='User (read only)'
    )
    is_active = models.BooleanField(
        default=True,
        verbose_name='Active'
    )
    waf_alias = models.TextField(
        blank=True,
        null=True,
        verbose_name='WAF ID'
    )
    waf_mode = models.BooleanField(
        default=False,
        verbose_name='Work mode'
    )
    combined = models.BooleanField(
        default=True,
        verbose_name='Cloud'
    )
    domain = models.TextField(
        blank=True,
        null=True,
        verbose_name='Nemesida WAF Cloud Domain'
    )
    acl = models.TextField(
        blank=True,
        null=True,
        verbose_name='ACL'
    )
    search_history = models.TextField(
        blank=True,
        null=True,
        verbose_name='Search history'
    )
    csv_report = models.BooleanField(
        default=False,
        verbose_name='Enable CSV Report'
    )
    pdf_report = models.BooleanField(
        default=False,
        verbose_name='Enable pdf Report'
    )
    mt_toggle = models.BooleanField(
        default=True,
        verbose_name='Enable MT'
    )
    oa_toggle = models.BooleanField(
        default=True,
        verbose_name='Enable OpenAPI'
    )
    ws_toggle = models.BooleanField(
        default=True,
        verbose_name='Enable WS'
    )
    ec_toggle = models.BooleanField(
        default=True,
        verbose_name='Enable EC'
    )
    last_update = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name='Last update'
    )
    objects = WAFUSERMANAGER()

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    def get_full_name(self):
        # The user is identified by their email address
        return self.email

    def get_short_name(self):
        # The user is identified by their email address
        return self.email

    @staticmethod
    def has_perm():
        """Does the user have a specific permission?"""
        # Simplest possible answer: Yes, always
        return True

    @staticmethod
    def has_module_perms():
        """Does the user have permissions to view the app `app_label`?"""
        # Simplest possible answer: Yes, always
        return True

    @property
    def is_staff(self):
        """Is the user a member of staff?"""
        # Simplest possible answer: All admins are staff
        return self.is_admin

    def is_readonly(self):
        return self.priv_readuser

    def get_role(self):
        if self.is_admin:
            return 2
        if self.priv_readuser:
            return 0
        return 0

    def set_role(self, role):
        if role == 0:
            self.is_admin = False
            self.priv_readuser = True
        elif role == 2:
            self.is_admin = True
            self.priv_readuser = False

    @property
    def is_superuser(self):
        # this param need to rosetta translate
        return self.is_admin
