import json
import requests
import logging
from django.conf import settings
from main.utils import get_proxy_api_dict
from url_normalize import url_normalize

try:
    api_url = settings.API_URI
except AttributeError:
    api_url = ''

USER_AGENT = settings.USER_AGENT
logger = logging.getLogger(__name__)


def send_request_get_target_list(key):

    # init
    target_list = []

    try:
        response = requests.post(
            url_normalize(api_url + '/get_target_list'),
            headers={'User-Agent': USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json={"key": key},
        )

    except Exception as e:
        logger.error(
            'An error occurred while fetching scanner data: {}'.format(e))
        return None
    if response.ok:
        try:
            # Response should be 'set' data
            data = json.loads(response.text)
            for i in data:
                target_list.append(i['scan_target'])
            return target_list
        except Exception as e:
            logger.error(
                'An error occurred while getting target list: {}'.format(e))
            return None
    else:
        logger.error('An error occurred while getting scanner data with response code: {}'.format(
            response.status_code))
        return None


def send_request_get_target_settings(key, scan__target):
    try:
        response = requests.post(
            url_normalize(api_url + '/get_target_setting'),
            headers={'User-Agent': USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json={"key": key},
        )

    except Exception as e:
        logger.error(
            'An error occurred while fetching scanner data: {}'.format(e))
        return None

    if response.ok:
        try:
            # Response should be 'set' data
            data = json.loads(response.text)
            for i in data:
                if i['scan__target'] == scan__target:
                    return i

        except Exception as e:
            logger.error(
                'An error occurred while getting target settings: {}'.format(e))
            return None
    else:
        logger.error('An error occurred while getting target settings with response code: {}'.format(
            response.status_code))
        return None


def send_request_run_scanning(key, start):
    data = {
        "key": key,
        "start": start
    }

    try:
        response = requests.post(
            url_normalize(api_url + '/run_scanning'),
            headers={'User-Agent': USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json=data,
        )
    except Exception as e:
        logger.error(
            'An error occurred while starting/stopping scanner: {}'.format(e))
        return None

    if response.ok:
        return True
    else:
        logger.error(
            'An error occurred while starting/stopping scanner with status code: {}'.format(response.status_code))
        return False


def send_request_del_target_settings(key, scan__target):
    data = {
        "key": key,
        "del": {
            "scan__target": scan__target
        }
    }

    try:
        response = requests.post(
            url_normalize(api_url + '/del_target_settings'),
            headers={'User-agent': USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json=data,
        )

    except Exception as e:
        logger.error(
            'An error occurred while deleting target settings: {}'.format(e))
        return None

    if response.ok:
        return True
    else:
        logger.error('An error occurred while deleting target settings with status code: {}'.format(response.status_code))
        return False


def send_request_set_target_settings(key, scan__target, scan__ssl, scan__proxy, auth__auth_path, auth__login_field,
                                     auth__login_value, auth__password_field, auth__password_value, modules):
    data = {
        "key": key,
        "set": {
            "scan__target": scan__target,
            "scan__ssl": scan__ssl,
            "scan__proxy": scan__proxy,
            "auth__auth_path": auth__auth_path,
            "auth__login_field": auth__login_field,
            "auth__login_value": auth__login_value,
            "auth__password_field": auth__password_field,
            "auth__password_value": auth__password_value,
            "modules": modules
        }
    }

    try:
        response = requests.post(
            url_normalize(api_url + '/set_target_settings'),
            headers={'User-agent': USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json=data,
        )
    except Exception as e:
        logger.error(
            'An error occurred while changing target settings: {}'.format(e))
        return None

    if response.ok:
        return True
    else:
        logger.error('An error occurred while changing target settings with status code: {}'.format(
            response.status_code))
        return False


def send_request_get_status_scanner(key):

    try:
        response = requests.post(
            url_normalize(api_url + '/get_status_scanner'),
            headers={'Use-agent': USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json={"key": key},
        )
    except Exception as e:
        logger.error(
            'An error occurred while getting scanner status: {}'.format(e))
        return None

    if response.ok:
        try:
            status = json.loads(response.text)
            return status
        except Exception as e:
            logger.error('An error occurred while parsing status data: {}'.format(e))
            return None
    else:
        logger.error('An error occurred while getting scanner status with status code: {}'.format(response.status_code))
        return None
