window.home = document.querySelector('#home').title;

function reducing_str(value, if_len=15, num=6) {
    if (value.length > if_len)
        return value.slice(0, num).concat('***').concat(value.slice(-num))
    return value
}

function parse(str) {
    let args = [].slice.call(arguments, 1),
        i = 0;

    return str.replace(/%s/g, () => args[i++]);
}

// for auto-loading pages
window.num_page = 1;
window.infinite = new Waypoint.Infinite({
    element: document.getElementById('myTable'),
    onBeforePageLoad: function () {
        $('.btn-loading-next-page').hide();
        $('.loading-container').show();

        $('.loading').show();
        performMark("{{ q }}");
        $('[data-toggle="tooltip"]').tooltip({
            trigger: 'hover',
        });
    },
    onAfterPageLoad: function ($items) {
        $('.loading-container').hide();
        $('.btn-loading-next-page').show();

        window.num_page += 1;

        $('.loading').hide();
        performMark("{{ q }}");
        $('[data-toggle="tooltip"]').tooltip({
            trigger: 'hover',
        });
    }
});

// loading pages by button
$(document).on('click', '.btn-loading-next-page', function (e) {
    if ($('.loading-container').is(':visible')){
        return;
    }
    $('.btn-loading-next-page').hide();
    $('.loading-container').show();
    let url_next_page = new URL(window.location);
    url_next_page.searchParams.set("page", window.num_page + 1);
    $.ajax({
        url: url_next_page,
        type: 'GET',
        success: function (data) {
            let context = $(data).find('tbody.context');
            let link_for_auto_loading = $(data).find('.infinite-more-link');
            if (context.length && context.children().length){
                $('tbody.context').append(context.children());
                window.num_page += 1;
                if (link_for_auto_loading.length){
                    $('#tableContent').after(link_for_auto_loading);
                }
                $('.btn-loading-next-page').show();
                $('.loading-container').hide();
            }else{
                $('.btn-loading-next-page').parent().hide();
            }
        }
    });
});


function sendData(d, url, p = 'main', reload = false) {
    var csrftoken = jQuery("[name=csrfmiddlewaretoken]").val();
    $.ajax({
        url: window.home + 'model_tuning/' + url,
        type: 'POST',
        data: {
            'data': d,
            csrfmiddlewaretoken: csrftoken,
            'table': p
        },
        dataType: 'json',
        success: function (data) {
            if (reload) {
                window.location.reload(true);
            }
        },
        error: function (response) {
            try {
                let data = JSON.parse(response.responseText);
                let block_messages = document.getElementById('messages-block');
                if (data['messages'] != null && data['messages'] != undefined) {
                    for (let message of data['messages']) {
                        let container = document.createElement('div');
                        container.className = 'container-fluid p-0';
                        let alert_div = '<div class="alert ' + message.tags + ' alert-dismissible" role="alert" style="margin-bottom: 10px">\n' +
                            '                <button type="button" class="close" data-dismiss="alert" aria-label="Close">\n' +
                            '                  <span aria-hidden="True">&times;</span>\n' +
                            '                </button>\n' +
                            '                ' + message.message + '\n' +
                            '              </div>'
                        $(container).html(alert_div);
                        block_messages.appendChild(container);

                    }
                }
            } catch (e) {

            }

        }
    });
}

function getCheckedBoxes(chkboxName, attr) {
    var checkboxes = document.getElementsByName(chkboxName);
    var checkboxesChecked = [];
    for (let item of checkboxes) {
        if (item.checked && item.parentElement.parentElement.style.display !== 'none') {
            checkboxesChecked.push(item[attr]);
        }
    }
    return checkboxesChecked.length > 0 ? checkboxesChecked : null;
}

function removeRows(rows) {
    if (rows != null) {
        for (var i = 0; i < rows.length; i++) {
            document.getElementById(rows[i]).parentElement.parentElement.remove()
        }
    }
}


function checkRow(iid) {
    var csrftoken = jQuery("[name=csrfmiddlewaretoken]").val();
    var row_id = document.getElementsByClassName('row-id')[0].childNodes[1].value;
    $.ajax({
        url: window.home + 'model_tuning/' + 'check/',
        type: 'POST',
        data: {
            'data': iid,
            csrfmiddlewaretoken: csrftoken
        },
        dataType: 'json',
        success: function (data) {
            if (data.row_count === 0) {
                document.getElementById(row_id).parentElement.parentElement.remove();
                return
            }
            $('#checkTable').find("tr:gt(0)").remove();
            if (Object.keys(data).length === 0) {
                document.getElementById('zoneContainer').innerHTML = "<p style='margin-bottom: 0px; margin-top: 20px; color:#1a5388; font-weight: 600;'>The request does not contain<br>any signature</p>";
                document.getElementById('checkTable').style.display = 'none';
                document.getElementById('checkModalDialog').style.maxWidth = "300px";
                setTimeout(function () {
                    $("#checkModal").modal('hide');
                }, 10000);
                document.getElementById('checkButton').style.display = 'none';
            } else {
                document.getElementById('checkTable').style.display = '';
                document.getElementById('checkButton').style.display = '';
                document.getElementById('checkModalDialog').style.maxWidth = "800px";
                for (a = 0; a < Object.keys(data).length; a++) {
                    var key = Object.keys(data)[a];
                    var val = Object.keys(data[key]);
                    dv = Array.from(new Set(data[key][val]));
                    document.getElementById('zoneContainer').innerHTML = "";
                    for (i = 0; i < dv.length; i++) {
                        rule_id = dv[i]['id'];
                        target = dv[i]['target'];
                        score_k = dv[i]['score_k'];
                        score_v = dv[i]['score_v'];
                        zone = key;
                        flags_val = dv[i]['flags'];
                        if (flags_val == 0) {
                            flags = gettext('No');
                        } else {
                            flags = gettext('Yes');
                        }
                        var r = Math.random().toString(36).substring(7);
                        $('#checkTable').find('tbody').append(`<tr style="font-size:80%"><td>${rule_id}</td><td>${zone}</td><td>${score_k}</td><td>${score_v}</td><td id="target_${r}_${rule_id}"></td><td>${flags}</td></tr>`);
                        $('#target_' + r + '_' + rule_id).text(target);
                    }
                }
            }
            $('#checkModal').modal('show');
        },
        error: function (response) {
            try {
                let data = JSON.parse(response.responseText);
                let block_messages = document.getElementById('messages-block');
                if (data['messages'] != null && data['messages'] != undefined) {
                    for (let message of data['messages']) {
                        let container = document.createElement('div');
                        container.className = 'container-fluid p-0';
                        let alert_div = '<div class="alert ' + message.tags + ' alert-dismissible" role="alert" style="margin-bottom: 10px">\n' +
                            '                <button type="button" class="close" data-dismiss="alert" aria-label="Close">\n' +
                            '                  <span aria-hidden="True">&times;</span>\n' +
                            '                </button>\n' +
                            '                ' + message.message + '\n' +
                            '              </div>'
                        $(container).html(alert_div);
                        block_messages.appendChild(container);

                    }
                }

            } catch (e) {

            }

        }
    });
}

//check row
$(document).on('click', '.cr', function (e) {
    var csrftoken = jQuery("[name=csrfmiddlewaretoken]").val();
    $(".pop").popover('hide');
    var row_id = document.getElementsByClassName('row-id')[0].childNodes[1].value;
    checkRow(row_id);
});

// create export
$(document).on('click', '.ct', function (e) {
    var row = e.currentTarget.parentElement.parentElement;
    var rt = row.getElementsByClassName("req_type")[0].innerHTML;
    var rb = row.getElementsByClassName("row_body");
    var mz = row.getElementsByClassName("row_mz")[0].innerHTML;
    if (typeof rb !== 'undefined' && rb.length != 0) {
        var b = rb[0].innerHTML;
    } else {
        var b = '';
    }
    var bt = row.getElementsByClassName("row_bt")[0].firstElementChild.innerText;
    if (bt == 11 && rt == 'GET') {
        document.getElementById('exportBody').style.display = 'none';
    } else {
        document.getElementById('exportBody').style.display = '';
    }
    document.getElementById('m_id').value = row.getElementsByClassName("row_id")[0].childNodes[1].id;
    document.getElementById('m_bt').value = row.getElementsByClassName("row_bt")[0].firstElementChild.innerHTML;
    if (bt == 12 || bt == 13 || (bt == 11 && mz)) {
        var obj = {};
        obj.row_id = document.getElementById('m_id').value;
        obj.bt = document.getElementById('m_bt').value;
        obj.fields = [];
        var myJSONText = JSON.stringify(obj);
        sendData(myJSONText, 'export/');
        document.getElementById(obj.row_id).parentElement.parentElement.remove()
    } else if (bt == 11) {
        $('#exampleModal').modal('show');
    }
});

// create modal export
$(document).on('click', '.ctm', function (e) {
    var row_id = document.getElementById('detailId').value;
    var ii = document.getElementById(row_id);
    var row = ii.parentElement.parentElement;
    var rt = row.getElementsByClassName("req_type")[0].innerHTML;
    var rb = row.getElementsByClassName("row_body");
    if (typeof rb !== 'undefined' && rb.length != 0) {
        var b = rb[0].innerHTML;
    } else {
        var b = '';
    }
    var bt = row.getElementsByClassName("row_bt")[0].firstElementChild.innerText;
    if (bt == 11 && (b == '' || rt == 'GET')) {
        document.getElementById('exportBody').style.display = 'none';
    } else {
        document.getElementById('exportBody').style.display = '';
    }
    document.getElementById('m_id').value = row.getElementsByClassName("row_id")[0].childNodes[1].id;
    document.getElementById('m_bt').value = row.getElementsByClassName("row_bt")[0].firstElementChild.innerHTML;
    if (bt == 12 || bt == 13) {
        var obj = {};
        obj.row_id = document.getElementById('m_id').value;
        obj.bt = document.getElementById('m_bt').value;
        obj.fields = [];
        var myJSONText = JSON.stringify(obj);
        sendData(myJSONText, 'export/');
        document.getElementById(obj.row_id).parentElement.parentElement.remove();
        $('#detailModal').modal('toggle');
    } else if (bt == 11) {
        $('#detailModal').modal('toggle');
        $('#exampleModal').modal('show');
    }
});

// check modal
$(document).on('click', '.chtm', function (e) {
    var csrftoken = jQuery("[name=csrfmiddlewaretoken]").val();
    var row_id = document.getElementById('detailId').value;
    $('#detailModal').modal('toggle');
    checkRow(row_id);
});


// export checked
$(document).on('click', '#ec', function (e) {
    var checkedBoxes = getCheckedBoxes("modalCheckBox", 'id');
    var bt = document.getElementById('m_bt').value;
    if (bt == 11 && checkedBoxes != null) {
        var arr = [];
        var obj = {};
        var i;
        for (i = 0; i < checkedBoxes.length; i++) {
            arr.push(checkedBoxes[i].split('_')[1]);
        }
        obj.row_id = document.getElementById('m_id').value;
        obj.fields = arr;
        obj.bt = document.getElementById('m_bt').value;
        var myJSONText = JSON.stringify(obj);
        sendData(myJSONText, 'export/');
        document.getElementById(obj.row_id).parentElement.parentElement.remove();
        // uncheck all
        var checkboxes = document.getElementsByName('modalCheckBox');
        for (var i = 0; i < checkboxes.length; i++) {
            if (checkboxes[i].type == 'checkbox') {
                checkboxes[i].checked = false;
            }
        }
        $('#exampleModal').modal('toggle');
    }
});

// check all
$(document).on('click', '#ca', function () {
    ca = document.getElementById('ca');
    var checktoggle = ca.checked;
    var checkboxes = document.getElementsByName('del_check');
    for (var i = 0; i < checkboxes.length; i++) {
        if (checkboxes[i].type == 'checkbox') {
            checkboxes[i].checked = checktoggle;
        }
    }
});

// delete row from main table
$(document).on('click', '.dtr', function (e) {
    $(".pop").popover('hide');
    var row = e.currentTarget.parentElement.parentElement;
    var row_id = row.getElementsByClassName('row_id')[0].childNodes[1].id;
    dda(row_id);
});

// delete row from modal window
$(document).on('click', '.dtmr', function (e) {
    $('#detailModal').modal('toggle');
    var row_id = document.getElementById('detailId').value;
    dda(row_id);
});

function dda(row_id) {
    var del_arr = [row_id];
    var url_string = window.location.href;
    var url = new URL(url_string);
    var c = url.searchParams.get("t");
    if (del_arr != null) {
        var myJSONText = JSON.stringify(del_arr);
        sendData(myJSONText, 'delete/', c);
        removeRows(del_arr);
        var rows = document.getElementById('myTable').getElementsByTagName("tr").length;
        if (rows == 1) {
            var content = document.getElementById('tableContent');
            // content.remove();
            $.ajax({
                url: window.location.href,
                headers: {
                    "Pragma": "no-cache",
                    "Expires": -1,
                    "Cache-Control": "no-cache"
                }
            }).done(function () {
                window.location.reload(true);
            });
        }
    }
}

// delete checked
$(document).on('click', '#dc', function () {
    var checkedBoxes = getCheckedBoxes("del_check", 'id');
    if (checkedBoxes != null) {
        document.getElementById('deleteConfirm').innerHTML = gettext("Delete record(s)?");
        $('#exampleModalCenter').modal('show');
    } else {
        window.del_all = true
        let urlParams = new URLSearchParams(window.location.search);
        urlParams.set('only_num', 'true');
        if (urlParams.get('t') == null){
            urlParams.set('t', 'm')
        }
        let url = window.location.href;
        if (url.includes('admin_panel')) {
            url = window.location.href.slice(0, url.indexOf('admin_panel'));
        } else {
            url = ''
        }
        $.ajax({
            url: url + 'search/?' + urlParams.toString(),
            type: 'GET',
            success: function (data) {
                let count = data.count;
                document.getElementById('deleteConfirm').innerHTML = parse(gettext("Delete all (%s) record(s)?"), count);
                $('#exampleModalCenter').modal('show');
        }
    });

    }
});

// delete all
$('#da').click(function () {
    var url_string = window.location.href;
    var url = new URL(url_string);
    var c = url.searchParams.get("t");
    var checkedBoxes = getCheckedBoxes("del_check", 'id');
    if (checkedBoxes != null) {
        var myJSONText = window.del_all ? 'all' : JSON.stringify(checkedBoxes) ;
        delete window.del_all;
        sendData(myJSONText, 'delete/', c, true);
        removeRows(checkedBoxes);
        var rows = document.getElementById('myTable').getElementsByTagName("tr").length;
        if (rows == 1) {
            var content = document.getElementById('tableContent');
            content.remove();
            $.ajax({
                url: window.location.href,
                headers: {
                    "Pragma": "no-cache",
                    "Expires": -1,
                    "Cache-Control": "no-cache"
                }
            }).done(function () {
                window.location.reload(true);
            });
        }
    } else {
        sendData('all', 'delete/', c);
        var content = document.getElementById('tableContent');
        content.remove();
        $.ajax({
            url: window.location.href,
            headers: {
                "Pragma": "no-cache",
                "Expires": -1,
                "Cache-Control": "no-cache"
            }
        }).done(function () {
            window.location.reload(true);
        });
    }
});

//escape html string
function escapeHtml(string) {
    let entityMap = {
        '&': '&amp;',
        '<': '&lt;',
        '>': '&gt;',
        '"': '&quot;',
        "'": '&#39;',
        '/': '&#x2F;',
        '`': '&#x60;',
        '=': '&#x3D;',
        "\n": '<br>'
    };

    return String(string).replace(/[&<>"'`=\/]|[\n]/g, function (s) {
        return entityMap[s];
    });
}

function replacing_first_match_string_in_text(text, pattern=/:/, repl_text=': '){
    const lines = text.split('\n');

    const modified_lines = lines.map(line => line.replace(pattern, repl_text));

    return modified_lines.join('\n');
}

//detail row
$(document).on('click', '.dr', function (e) {
    $(".pop").popover('hide');
    var row_id = document.getElementsByClassName('row-id')[0].childNodes[1].value;
    var params = window.location.search.substr(1);
    var url = row_id + "?" + params;
    $.get(window.home + 'model_tuning/' + url)
        .done(function (data) {
            if (data.row_count === 0) {
                document.getElementById(row_id).parentElement.parentElement.remove();
                return
            }

            if (data.bt === 11 || data.bt === 13) {
                $('#detailExport').hide()
            } else {
                $('#detailExport').show()
            }
            var arr = ['args', 'uri', 'cookie', 'ua', 'body', 'referer', 'other_headers'];
            document.getElementById('detailId').value = row_id;
            document.getElementById('detail_row_bt').innerHTML = data.bt;
            if (typeof data.ip !== 'undefined') {
                document.getElementById('detail_row_info').innerHTML = data.ip + '<br>' + data.vhost + '<br>' + data.method + ', ' + data.agent + ', ' + data.waf_id + '<br>' + data.timestamp + '<br>' + data.request_id;
            } else {
                document.getElementById('detail_row_info').innerHTML = data.host + '<br>' + data.req_t + '<br>' + data.id + '<br>' + data.date;
            }
            $("#detail_row_args").html(escapeHtml(data.args));
            $("#detail_row_uri").html(escapeHtml(data.uri));
            $("#detail_row_cookie").html(escapeHtml(data.cookie));
            $("#detail_row_ua").html(escapeHtml(data.ua));

            $("#detail_row_body").html(escapeHtml(data.body));
            $("#detail_row_referer").html(escapeHtml(data.referer));

            $("#detail_row_other_headers").html(escapeHtml(replacing_first_match_string_in_text(data.other_headers, /:/, ": ")));
            for (var i = 0; i < arr.length; i++) {
                if (data[arr[i]] == '' || data[arr[i]] == '-' || data[arr[i]] == null || data[arr[i]] == " ") {
                    document.getElementById('detail_row_' + arr[i]).style.display = 'none';
                    document.getElementById('detail_head_' + arr[i]).style.display = 'none';
                } else {
                    document.getElementById('detail_row_' + arr[i]).style.display = '';
                    document.getElementById('detail_head_' + arr[i]).style.display = '';
                }
            }
            $('#detailModal').modal('show');
        });
});

$("#searchForm").submit(function (eventObj) {
    var url_string = window.location.href;
    var url = new URL(url_string);
    var c = url.searchParams.get("t");
    $('<input />').attr('type', 'hidden')
        .attr('name', "t")
        .attr('value', c)
        .appendTo('#searchForm');
    return true;
});

$(document).on('click', '.edr', function (e) {
    $('#detailModal').modal('toggle');
    var row_id = document.getElementById('detailId').value;
    var params = window.location.search.substr(1);
    var url = "/" + row_id + "?" + params;
    $.get(url)
        .done(function (data) {
            $('#editID').val(row_id);
            $('#editHost').val(data.host);
            $('#editUri').val(data.uri);
            $('#editArgs').val(data.args);

            $('#editBody').val(data.body);
            $('#editUA').val(data.ua);
            $('#editCookie').val(data.cookie);
            $('#editContt').val(data.cont_t);
            $('#editRef').val(data.referer);
            if ($('#editUri').val().length == 0) {
                $('.edit-uri').hide();
            } else {
                $('.edit-uri').show();
            }
            if ($('#editArgs').val().length == 0) {
                $('.edit-args').hide();
            } else {
                $('.edit-args').show();
            }
            if ($('#editBody').val().length == 0) {
                $('.edit-body').hide();
            } else {
                $('.edit-body').show();
            }
            if ($('#editUA').val().length == 0) {
                $('.edit-ua').hide();
            } else {
                $('.edit-ua').show();
            }
            if ($('#editCookie').val().length == 0) {
                $('.edit-cookie').hide();
            } else {
                $('.edit-cookie').show();
            }
            if ($('#editContt').val().length == 0) {
                $('.edit-contt').hide();
            } else {
                $('.edit-contt').show();
            }
            if ($('#editRef').val().length == 0) {
                $('.edit-ref').hide();
            } else {
                $('.edit-ref').show();
            }
            if ($('#editHeaders').val().length == 0) {
                $('.edit-headers').hide();
            } else {
                $('.edit-headers').show();
            }
            $('#editModal').modal('show');
        });
});


$(document).on('click', '.ednr', function (e) {
    $('#detailModal').modal('toggle');
    var row_id = document.getElementById('detailId').value;
    var params = window.location.search.substr(1);
    var url = "/" + row_id + "?" + params;
    document.getElementById('bt-edit-form').action = '/editfn/';
    $.get(window.home + 'model_tuning/' + url)
        .done(function (data) {

            $('#editID').val(row_id);
            $('#editHost').val(data.host);
            $('#editUri').val(data.uri);
            $('#editArgs').val(data.args);
            $('#editBody').val(data.body);
            $('#editUA').val(data.ua);
            $('#editCookie').val(data.cookie);
            $('#editContt').val(data.cont_t);
            $('#editRef').val(data.referer);
            $('#editHeaders').val(data.other_headers);
            if ($('#editUri').val().length == 0) {
                $('.edit-uri').hide();
            } else {
                $('.edit-uri').show();
            }
            if ($('#editArgs').val().length == 0) {
                $('.edit-args').hide();
            } else {
                $('.edit-args').show();
            }
            if ($('#editBody').val().length == 0) {
                $('.edit-body').hide();
            } else {
                $('.edit-body').show();
            }
            if ($('#editUA').val().length == 0) {
                $('.edit-ua').hide();
            } else {
                $('.edit-ua').show();
            }
            if ($('#editCookie').val().length == 0) {
                $('.edit-cookie').hide();
            } else {
                $('.edit-cookie').show();
            }
            if ($('#editContt').val().length == 0) {
                $('.edit-contt').hide();
            } else {
                $('.edit-contt').show();
            }
            if ($('#editRef').val().length == 0) {
                $('.edit-ref').hide();
            } else {
                $('.edit-ref').show();
            }
            if ($('#editHeaders').val().length == 0) {
                $('.edit-headers').hide();
            } else {
                $('.edit-headers').show();
            }
            $('#editModal').modal('show');
        });
});

$(document).on('click', '.er', function (e) {
    try {
        var row_id = document.getElementsByClassName('row-id')[0].childNodes[1].value;
    } catch (err) {
        var row = e.currentTarget.parentElement.parentElement;
        var row_id = row.getElementsByClassName('row_id')[0].childNodes[1].id;
    }
    var params = window.location.search.substr(1);
    var url = '';
    if (params === '') {
        url = "/" + row_id + "?edit=true"
    } else {
        url = "/" + row_id + "?" + params + '&edit=true'
    }
    $.get(window.home + 'model_tuning/' + url)
        .done(function (data) {
            if (data.row_count === 0) {
                document.getElementById(row_id).parentElement.parentElement.remove();
                return
            }
            $('#editID').val(row_id);
            $('#editUri').val(data.uri);
            $('#editArgs').val(data.args);
            $('#editBody').val(data.body);
            $('#editUA').val(data.ua);
            $('#editCookie').val(data.cookie);
            $('#editRef').val(data.referer);
            $('#editHeaders').val(data.other_headers);
            var arr = data.mz.split("|");
            //hiding empty inputs
            if ($('#editUri').val().length == 0 || arr.includes('uri') == false) {
                $('.edit-uri').hide();
            } else {
                $('.edit-uri').show();
            }
            if ($('#editBody').val().length == 0 || arr.includes('body') == false) {
                $('.edit-body').hide();
            } else {
                $('.edit-body').show();
            }
            if ($('#editArgs').val().length == 0 || arr.includes('args') == false) {
                $('.edit-args').hide();
            } else {
                $('.edit-args').show();
            }
            if ($('#editUA').val().length == 0 || arr.includes('ua') == false) {
                $('.edit-ua').hide();
            } else {
                $('.edit-ua').show();
            }
            if ($('#editCookie').val().length == 0 || arr.includes('cookie') == false) {
                $('.edit-cookie').hide();
            } else {
                $('.edit-cookie').show();
            }
            if ($('#editRef').val().length == 0 || arr.includes('referer') == false) {
                $('.edit-ref').hide();
            } else {
                $('.edit-ref').show();
            }
            if ($('#editHeaders').val().length == 0 || arr.includes('other_headers') == false) {
                $('.edit-headers').hide();
            } else {
                $('.edit-headers').show();
            }
            $('#editModal').modal('show');
        });
});


$(document).on('click', '.enr', function (e) {
    try {
        var row_id = document.getElementsByClassName('row-id')[0].childNodes[1].value;
    } catch (err) {
        var row = e.currentTarget.parentElement.parentElement;
        var row_id = row.getElementsByClassName('row_id')[0].childNodes[1].id;
    }
    document.getElementById('bt-edit-form').action = '/editfn/';
    var params = window.location.search.substr(1);
    var url = "/" + row_id + "?" + params;
    $.get(window.home + 'model_tuning/' + url)
        .done(function (data) {
            $('#editID').val(row_id);
            $('#editUri').val(data.uri);
            $('#editArgs').val(data.args);
            $('#editBody').val(data.body);
            $('#editUA').val(data.ua);
            $('#editCookie').val(data.cookie);
            $('#editRef').val(data.referer);
            $('#editHeaders').val(data.other_headers);
            // var arr = data.mz.split("|");
            //hiding empty inputs
            if ($('#editUri').val().length == 0 || data.uri === null) {
                $('.edit-uri').hide();
            } else {
                $('.edit-uri').show();
            }
            if ($('#editBody').val().length == 0 || data.body === null) {
                $('.edit-body').hide();
            } else {
                $('.edit-body').show();
            }
            if ($('#editArgs').val().length == 0 || data.args === null) {
                $('.edit-args').hide();
            } else {
                $('.edit-args').show();
            }
            if ($('#editUA').val().length == 0 || data.ua === null) {
                $('.edit-ua').hide();
            } else {
                $('.edit-ua').show();
            }
            if ($('#editCookie').val().length == 0 || data.cookie === null) {
                $('.edit-cookie').hide();
            } else {
                $('.edit-cookie').show();
            }
            if ($('#editRef').val().length == 0 || data.referer === null) {
                $('.edit-ref').hide();
            } else {
                $('.edit-ref').show();
            }
            if ($('#editHeaders').val().length == 0 || data.other_headers === null) {
                $('.edit-headers').hide();
            } else {
                $('.edit-headers').show();
            }
            $('#editModal').modal('show');
        });
});


$(document).on('click', '.custom_check', function (e) {
    var obj = {};
    obj.uri = document.getElementById('customUri').value;
    obj.args = document.getElementById('customArgs').value;
    obj.other_headers = document.getElementById('customHeaders').value;
    obj.body = document.getElementById('customBody').value;
    var myJSONText = JSON.stringify(obj);
    var csrftoken = jQuery("[name=csrfmiddlewaretoken]").val();
    $.ajax({
        url: window.home + 'model_tuning/' + 'custom_check/',
        type: 'POST',
        data: {
            'data': myJSONText,
            csrfmiddlewaretoken: csrftoken,
        },
        dataType: 'json',
        success: function (data) {
            $('#checkTable').find("tr:gt(0)").remove();
            if (Object.keys(data).length === 0) {
                document.getElementById('zoneContainer').innerHTML = "<p style='margin-bottom: 0px; margin-top: 20px; color:#1a5388; font-weight: 600;'>The request  does not contain any signature</p>";
                document.getElementById('checkTable').style.display = 'none';
                document.getElementById('checkModalDialog').style.maxWidth = "300px";
                setTimeout(function () {
                    $("#checkModal").modal('hide');
                    $('#customCheckModal').modal('show');
                }, 10000);
                document.getElementById('checkButton').style.display = 'none'
            } else {
                document.getElementById('checkTable').style.display = '';
                document.getElementById('checkButton').style.display = '';
                document.getElementById('checkModalDialog').style.maxWidth = "800px";
                for (a = 0; a < Object.keys(data).length; a++) {
                    var key = Object.keys(data)[a];
                    var val = Object.keys(data[key]);
                    dv = Array.from(new Set(data[key][val]));
                    document.getElementById('zoneContainer').innerHTML = "";
                    for (i = 0; i < dv.length; i++) {
                        rule_id = dv[i]['id'];
                        target = dv[i]['target'];
                        score_k = dv[i]['score_k'];
                        score_v = dv[i]['score_v'];
                        zone = key;
                        flags_val = dv[i]['flags'];
                        if (flags_val == 0) {
                            flags = gettext('No');
                        } else {
                            flags = gettext('Yes');
                        }
                        var r = Math.random().toString(36).substring(7);
                        $('#checkTable').find('tbody').append(`<tr style="font-size:80%"><td>${rule_id}</td><td>${zone}</td><td>${score_k}</td><td>${score_v}</td><td id="target_${r}_${rule_id}"></td><td>${flags}</td></tr>`);
                        $('#target_' + r + '_' + rule_id).text(target);
                    }
                }
            }
            $('#customCheckModal').modal('toggle');
            var button = document.getElementById("checkButton");
            var clickFunction = function (event) {
                $('#customCheckModal').modal('show');
                button.removeEventListener('click', clickFunction, false);
            };
            button.addEventListener("click", clickFunction, false);
            $('#checkModal').modal('show');
        },
        error:
            function (response) {
                try {
                    let data = JSON.parse(response.responseText);
                    let block_messages = document.getElementById('messages-block');
                    if (data['messages'] != null && data['messages'] != undefined) {
                        for (let message of data['messages']) {
                            let container = document.createElement('div');
                            container.className = 'container-fluid p-0';
                            let alert_div = '<div class="alert ' + message.tags + ' alert-dismissible" role="alert" style="margin-bottom: 10px">\n' +
                                '                <button type="button" class="close" data-dismiss="alert" aria-label="Close">\n' +
                                '                  <span aria-hidden="True">&times;</span>\n' +
                                '                </button>\n' +
                                '                ' + message.message + '\n' +
                                '              </div>'
                            $(container).html(alert_div);
                            block_messages.appendChild(container);

                        }
                    }

                } catch (e) {

                }

            }
    });
});


function remove_all_by_ip(ip,elem) {
    let element = $(elem).parent().parent().parent().parent()[0]
    let data = $(element).find('*[data-id]')[0]
    ip = $(data).attr('data-id') // Todo: need fix

    let confirm = document.querySelector('#confirmModal');
    let b = confirm.querySelector('.confirm_btn');
    let b_close = confirm.querySelector('.close_btn');
    b.classList.add('yes_remove_by_ip');
    b.innerHTML = gettext('Yes');
    b_close.innerHTML = gettext('No');
    b_close.classList.remove('m_btn');
    b_close.classList.remove('btn-primary');
    b_close.classList.add('btn-default')
    confirm.querySelector('#bodyConfirm').innerHTML = parse(gettext('Remove all entries with an IP address <code>%s</code>?'), ip);
    $(confirm).modal('show');
    window.data_remove_by_ip = {"ip": ip};
}

$(document).on('click', '.yes_remove_by_ip', function (e) {
    let confirm = document.querySelector('#confirmModal');
    let b = confirm.querySelector('.confirm_btn');
    let b_close = confirm.querySelector('.close_btn');
    let csrftoken = jQuery("[name=csrfmiddlewaretoken]").val();

    b.innerHTML = gettext('Yes');
    b_close.innerHTML = gettext('No');
    $.ajax({
        url: home + 'model_tuning/remove_all_by_ip/',
        headers: {'X-CSRFToken': csrftoken},
        type: "POST",
        dataType: "html",
        data: window.data_remove_by_ip,
        success: function (response, textStatus, xhr) {
            if (xhr.status == 255) {
                show_messages_from_response(response);
                return;
            }

            let list_rows = document.querySelectorAll("#tableContent tbody tr")
            let ip = data_remove_by_ip['ip'];

            for (let i of list_rows) {
                if (i.querySelector(".contain_ip").querySelector(`[data-id="${ip}"]`) !== null) {
                    i.remove();
                }
            }

            b_close.classList.add('m_btn');
            b_close.classList.add('btn-primary');
            b_close.classList.remove('btn-default');
            b.classList.remove('yes_remove_by_ip');
            window.data_remove_by_ip = {};
        },
        error: function (response) {
            show_messages_from_response(response.responseText)
        }
    });
});

function showBlockBtn(element) {
    let hiddenText = $(element).find(".blockBtn")
    if (hiddenText.length !== 0) {
        let globe = $(element).find("#flag")
        $(hiddenText).css('visibility', 'unset')
        $(globe).css('margin-right', '0')
    }
}

function hideBlockBtn(element) {
    let hiddenText = $(element).find(".blockBtn")
    if (hiddenText.length !== 0) {
        let globe = $(element).find("#flag")
        $(hiddenText).css('visibility', "hidden")
        $(globe).css('margin-right', '-10px')
    }
}

$(document).on("mouseover", ".contain_ip", function () {
    showBlockBtn(this)
})
$(document).on("mouseout", ".contain_ip", function () {
    hideBlockBtn(this)
})
