import os
import time
import logging
import tempfile
import schedule

from pymemcache.client.base import Client
from pymemcache import MemcacheUnexpectedCloseError

from django.conf import settings

# Init memcached
cache_ip, cache_port = settings.CACHES['default']['LOCATION'].split(':')
cache_address = (cache_ip, int(cache_port))
mcache = Client(cache_address)

logger = logging.getLogger(__name__)


def clear_old_files():
    logger.info('Cleaning PDF reports')

    try:
        base_path = tempfile.gettempdir() + os.path.sep + 'nwaf-cabinet'
        reports_path = base_path + os.path.sep + 'reports'
        filenames = next(os.walk(reports_path), (None, None, []))[2]
        for file in filenames:

            if mcache.get(file) is None:
                os.remove(reports_path + os.path.sep + file)

    except MemcacheUnexpectedCloseError:
        logger.error('An error occurred while removing old files reports: MemcacheUnexpectedCloseError')

    except Exception as e:
        logger.error('An error occurred while removing old files reports: {}'.format(e))


if __name__ == '__main__':
    logger.info('Running cron tasks')
    schedule.every().day.at("00:00").do(clear_old_files)
    try:
        while True:
            schedule.run_pending()
            time.sleep(1)

    finally:
        schedule.clear()
        logger.info('Stopping the execution of crown tasks')
