
from django.contrib.auth.decorators import login_required
from django.http.response import JsonResponse
from django.views.decorators.http import require_POST
from django.contrib import messages
from django.shortcuts import render, redirect
from django.utils.translation import pgettext

from main.decorators import time_of_function
from main.models.my_waf import LicKeys
from main.scanner_utils import send_request_del_target_settings
from main.scanner_utils import send_request_get_status_scanner
from main.scanner_utils import send_request_get_target_list
from main.scanner_utils import send_request_get_target_settings
from main.scanner_utils import send_request_run_scanning
from main.scanner_utils import send_request_set_target_settings
from main.scanner_utils import settings
from main.utils import get_list_messages
from utils.db import check_db

new_table_exist = check_db(None)


@require_POST
@login_required
@time_of_function
def del_target_settings(request):
    key = request.POST.get('key')
    key = LicKeys.objects.get(id=int(key)).key
    scan__target = request.POST.get('scan__target')
    response = send_request_del_target_settings(key, scan__target)
    if response:
        return JsonResponse(data={'status': 'ok'})
    else:
        messages.error(request, pgettext('messages', "Something's went wrong"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)

        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)


@require_POST
@login_required
@time_of_function
def run_scanning(request):
    key = request.POST.get('key')
    key = LicKeys.objects.get(id=int(key)).key
    start = request.POST.get('start')
    response = send_request_run_scanning(key, start)
    if response:
        return JsonResponse(data={'status': 'ok'})
    else:
        messages.error(request, pgettext('messages', "Something's went wrong"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)

        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)


@require_POST
@login_required
@time_of_function
def set_target_settings(request):
    key = request.POST.get('key')
    key = LicKeys.objects.get(id=int(key)).key
    scan__target = request.POST.get('scan__target')
    scan__ssl = request.POST.get('scan__ssl')
    scan_proxy = request.POST.get('scan_proxy')
    auth__auth_path = request.POST.get('auth__auth_path')
    auth__login_field = request.POST.get('auth__login_field')
    auth__login_value = request.POST.get('auth__login_value')
    auth__password_field = request.POST.get('auth__password_field')
    auth__password_value = request.POST.get('auth__password_value')
    modules = request.POST.getlist('modules')
    response = send_request_set_target_settings(
        key, scan__target, scan__ssl, scan_proxy, auth__auth_path,
        auth__login_field, auth__login_value, auth__password_field,
        auth__password_value, modules
    )
    if response:
        return JsonResponse(data={'status': 'ok'})
    else:
        messages.error(request, pgettext('messages', "Something's went wrong"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)

        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)


@login_required
@time_of_function
def get_target_list(request):
    if not new_table_exist or not settings.API_URI:
        return redirect('index')

    key = request.POST.get('key')
    key = LicKeys.objects.get(id=int(key)).key

    target_list = send_request_get_target_list(key)
    if not target_list:
        messages.error(request, pgettext('messages', "Something's went wrong"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)

        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)

    return render(request, 'main/scanner_settings.html', {'target_list': target_list})


@login_required
@time_of_function
def get_target_settings(request):
    if not new_table_exist or not settings.API_URI:
        return redirect('index')

    key = request.POST.get('key')
    key = LicKeys.objects.get(id=int(key)).key
    scan__target = request.POST.get('scan__target')

    target_settings = send_request_get_target_settings(key, scan__target)
    if not target_settings:
        messages.error(request, pgettext('messages', "Something's went wrong"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)

        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)

    context = {
        "scan__target": target_settings["scan__target"],
        "scan__ssl": target_settings["scan__ssl"],
        "scan__proxy": target_settings["scan__proxy"],
        "auth__auth_path": target_settings["auth__auth_path"],
        "auth__login_field": target_settings["auth__login_field"],
        "auth__login_value": target_settings["auth__login_value"],
        "auth__password_field": target_settings["auth__password_field"],
        "auth__password_value": target_settings["auth__password_value"],
        "modules": target_settings["modules"]
    }

    return render(request, 'main/scanner_settings.html', context)


@require_POST
@login_required
@time_of_function
def get_status_scanner(request):
    if not new_table_exist or not settings.API_URI:
        return redirect('index')

    key = request.POST.get('key')
    key = LicKeys.objects.get(id=int(key)).key
    status = send_request_get_status_scanner(key)
    if not status:
        messages.error(request, pgettext('messages', "Something's went wrong"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)

        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)
    return render(request, 'main/scanner_settings.html', status)
