import ipaddress
import re
from django import template

register = template.Library()


def local(ip):
    try:
        addr = ipaddress.ip_address(ip)
        if addr.is_private:
            return True
    except Exception:
        reg = re.compile(
            r"(^127\.)|(^10\.)|(^172\.1[6-9]\.)|(^172\.2\d\.)|(^172\.3[0-1]\.)|(^192\.168\.)|"
            r"(^fc00)|(^FC00)|(^fd)|(^FD)"
        )
        if reg.match(ip):
            return True
        return False


def multicast(ip):
    reg = re.compile(
        r"(^22[4-9]\.)|(^23\d\.)")
    if reg.match(ip):
        return True
    return False


@register.filter
def local_ip(value):
    if isinstance(value, str):
        if local(value):
            return 'local'
        elif multicast(value):
            return 'multicast'
    else:
        return 'public'
