import re
import os
import logging
import requests
import ipaddress

from datetime import datetime

from django.conf import settings
from main.utils import convert_from_punycode, get_proxy_conf_dict

logger = logging.getLogger(__name__)


def get_update_time():
    headers = {'User-Agent': settings.USER_AGENT}
    try:
        r = requests.get(settings.URI_RL_TS, headers=headers, proxies=get_proxy_conf_dict(), timeout=30)
        return int(r.text)
    except Exception as e:
        logger.error(
            'An error occurred while getting information about the timestamp of the rules. Error information: {err}'
            .format(err=e)
        )
        return None


def remove_zero_from_ip(ip):
    if type(ipaddress.ip_address(ip)) is ipaddress.IPv4Address:
        no_zero_ip = re.sub(r"\b0+(\d)", r'\1', str(ip))
    else:
        no_zero_ip = re.sub(r"\b0+(\d)", r'\1', str(ip))
    return str(no_zero_ip)


def split_zone_logs(logs_mt):
    for row in logs_mt:
        if type(row.mz) is str:
            row.mz = [i for i in row.mz.split('|') if i != '']
    return logs_mt


def format_logs_mt(logs_mt):
    BT_CELL_CLASS_MAPPING = {
        11: 'bt-red',
        12: 'bt-blue',
        13: 'bt-green',
    }

    BT_CELL_DEFAULT_CLASS = 'bt-black'

    for row in logs_mt:
        row.vhost = convert_from_punycode(row.vhost)
        if row.timestamp:
            row.timestamp = datetime.fromtimestamp(
                int(row.timestamp)).strftime('%H:%M:%S %d.%m.%Y')
        if row.other_headers:
            row.other_headers = os.linesep.join([i.strip() for i in row.other_headers.split('; ')])
            row.other_headers = row.other_headers.strip('; ')
        bt_value = getattr(row, 'bt', None)
        row.bt_class = ''
        if row.body:
            row.body = os.linesep.join([s for s in row.body.splitlines() if s])
        if bt_value is not None:
            row.bt_class = BT_CELL_CLASS_MAPPING.get(
                bt_value, BT_CELL_DEFAULT_CLASS
            )

    return logs_mt
