import base64
import json
import logging
import requests
from django.conf import settings
from django.shortcuts import render
from django.db import connections
from django.views.decorators.http import require_POST
from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.contrib import messages
from url_normalize import url_normalize

from django.utils.translation import pgettext, gettext as _

from main.decorators import time_of_function
from main.models.my_waf import LicKeys
from main.utils import get_list_messages
from main.utils import get_proxy_api_dict
from main.utils import get_proxy_conf_dict
from my_waf.utils import check_keys
from my_waf.utils import get_waf_id
from nginx.utils import set_nginx_conf
from utils.db import check_db

logger = logging.getLogger(__name__)

new_table_exist = check_db(logger)

try:
    api_url = settings.API_URI
except AttributeError:
    api_url = 'http://localhost:8080/nw-api/'

cache_ip, cache_port = settings.CACHES['default']['LOCATION'].split(':')
cache_address = (cache_ip, int(cache_port))

PAGE_SIZE = settings.DEFAULT_PAGE_SIZE

vGeoIP = settings.V_GEOIP
vAPP = settings.V_APP

isCCEnable = None
try:
    r = "SELECT * FROM information_schema.columns WHERE column_name = 'cc' AND table_name = 'attack'"
    with connections['waf'].cursor() as cursor:
        cursor.execute(r)
        res_cc = cursor.fetchone()
        if res_cc is None:
            isCCEnable = False
        else:
            isCCEnable = True

except Exception:
    isCCEnable = False


def get_lickeys(request):
    keys = LicKeys.objects.filter(user__id=request.user.id).all()
    lic_keys, checked_ids = check_keys(keys, check_waf_id=True)
    list_key_for_requests = [i for i in lic_keys if i.tarif == 'Enterprise']
    list_for_template = list_key_for_requests
    key_query_get = request.GET.get('key_query_id')
    key_query_post = request.POST.get('key_query_id')

    if key_query_get:
        list_key_for_requests = LicKeys.objects.filter(id=key_query_get)
    if key_query_post:
        list_key_for_requests = LicKeys.objects.filter(id=key_query_post)
    return lic_keys, list_key_for_requests, list_for_template


def api_vhost_only_query_func(list_key_for_requests: list):
    try:
        answer = requests.post(
            url_normalize(api_url + '/nginx/get_vhost'),
            headers={'User-agent': settings.USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            data={'key': list_key_for_requests[0].key},
        )
        if answer.ok:
            return answer

    except Exception as error:
        logger.error(f'An error occurred error when get nginx/get_vhost {error}')


def get_from_api_list_ssl(list_key_for_requests: list):
    try:
        answer = requests.post(
            url_normalize(api_url + '/nginx/get_certificate'),
            headers={'User-agent': settings.USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            data={'key': list_key_for_requests[0].key},
        )
        if answer.ok:
            answer = json.loads(answer.content.decode())
            return answer

    except Exception as error:
        logger.error(f'An error occurred error when get /nginx/get_certificate {error}')


def set_param_ssl(data: dict) -> dict:
    try:
        answer = requests.post(
            url_normalize(api_url + '/nginx/set_certificate'),
            headers={'User-agent': settings.USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json=data,
        )
        answer = json.loads(answer.content.decode())
        return answer

    except Exception as error:
        logger.error(f'An error occurred error when get /nginx/set_certificate {error}')


def set_ssl_config(request: object, list_key_for_requests: list) -> JsonResponse:
    if 'data' in request.POST:
        data_request = json.loads(request.POST['data'])
        cert_key = data_request['certificate_key'].replace('"', '')
        cert = data_request['certificate'].replace('"', '')
        cert_ssl_split = '\n'.join(cert.split('\\n'))
        cert_key_ssl_split = '\n'.join(cert_key.split('\\n'))
        data_request['certificate_key'] = base64.b64encode(cert_key_ssl_split.encode()).decode()
        data_request['certificate'] = base64.b64encode(cert_ssl_split.encode()).decode()
        data = {
            'key': list_key_for_requests[0].key,
            'set': data_request
        }
        answer = set_param_ssl(data)
        return answer


def decode_base64_ssl(answer_ssl):
    if 'configuration' in answer_ssl:
        for i in answer_ssl['configuration']:
            cert = base64.b64decode(i['certificate']).decode()
            cert_key = base64.b64decode(i['certificate_key']).decode()
            i['certificate'] = '\\n'.join(cert[27:len(cert) - 27].split('\\n'))
            i['certificate_key'] = '\\n'.join(cert_key[27:len(cert_key) - 27].split('\\n'))
    else:
        answer_ssl = {'status': 'none'}
    return answer_ssl


def set_dhparam_nginx(data: dict):
    answer = requests.post(
        url_normalize(api_url + '/nginx/set_dhparam'),
        headers={'User-agent': settings.USER_AGENT},
        proxies=get_proxy_api_dict(),
        timeout=30,
        json=data,
    )
    return answer


def get_dh_param(list_key_for_requests):
    try:
        answer = requests.post(
            url_normalize(api_url + '/nginx/get_dhparam'),
            headers={'User-agent': settings.USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            data={'key': list_key_for_requests[0].key},
        )
        answer = json.loads(answer.content.decode())
        return answer

    except Exception as error:
        logger.error(f'An error occurred error when get /nginx/get_dh_param {error}')


def create_param_for_api_dhparam(list_change: dict):
    text_dhparam = '\n'.join(list_change['dhparam'].replace('"', '').split('\\n'))
    base_64_text_dhparam = base64.b64encode(text_dhparam.encode()).decode()
    list_change['dhparam'] = base_64_text_dhparam
    return list_change


def edit_dhparam(request, list_key_for_requests):
    list_change = json.loads(request.POST['dh_set'])
    list_for_api = create_param_for_api_dhparam(list_change)
    data = {
        'key': list_key_for_requests[0].key,
        'set': list_for_api
    }
    answer = set_dhparam_nginx(data)
    return answer


def get_dhparam_for_templates(list_key_for_requests):
    dh_param = get_dh_param(list_key_for_requests)
    if dh_param['status'] == 'none':
        dh_param = {'configuration': []}

    for value in dh_param['configuration']:
        dh = base64.b64decode(value["dhparam"]).decode()
        value["dhparam"] = '\\n'.join(dh[29:len(dh) - 27].split('\\n'))
    return dh_param


@login_required
@time_of_function
def main_ssl(request, var=None):
    lic_keys, list_key_for_requests, list_for_template = get_lickeys(request)
    delete = request.GET.get('del')
    del_dhparam = request.GET.get('del_dhparam')

    if del_dhparam:
        data = {
            "key": list_key_for_requests[0].key,
            "del": {'name': del_dhparam}
        }

        set_dhparam_nginx(data)

    if 'dh_set' in request.POST:
        answer = edit_dhparam(request, list_key_for_requests)
        answer = json.loads(answer.content.decode())
        if answer['status'] != "success":
            return JsonResponse(data={'messages': answer['description'], 'status': 'error'}, status=255)
    dh_param = get_dhparam_for_templates(list_key_for_requests)

    if delete:
        data = {
            "key": list_key_for_requests[0].key,
            "del": {'name': delete}
        }
        set_param_ssl(data)

    if 'data' in request.POST:
        answer = set_ssl_config(request, list_key_for_requests)
        if answer['status'] != "success":
            return JsonResponse(data={'messages': answer['description'], 'status': 'error'}, status=255)

    answer_ssl = get_from_api_list_ssl(list_key_for_requests)
    answer_vhost = api_vhost_only_query_func(list_key_for_requests)
    answer_vhost = json.loads(answer_vhost.content.decode())
    answer_ssl = decode_base64_ssl(answer_ssl)

    if 'configuration' in answer_vhost and 'configuration' in answer_ssl:
        for i in answer_vhost['configuration']:
            for ssl in answer_ssl['configuration']:
                if i['certificate_name'] == ssl['name']:
                    ssl['ssl_ciphers'] = i['ssl_ciphers']
                    ssl['dhparam_name'] = i['dhparam_name']
                    ssl['ssl_protocols'] = i['ssl_protocols']
                    ssl['certificate_name'] = i['certificate_name']
                    ssl['ssl_session_timeout'] = i['ssl_session_timeout']

    else:
        answer_vhost['configuration'] = []
        answer_ssl['configuration'] = []
    if not answer_ssl['configuration']:
        answer_ssl = {'configuration': []}
    list_ssl_config = [i['name'] for i in answer_ssl['configuration']]

    context = {
        'list_ssl_config': list_ssl_config,
        'list_ssl': answer_ssl['configuration'],
        'dh_param': dh_param['configuration']
    }
    if var:
        return answer_ssl['configuration'], list_ssl_config
    else:
        return render(request, 'nginx/main_ssl.html', context)


def set_list_upstreams_from_api(data: dict) -> object:
    answer = requests.post(
        url_normalize(api_url + '/nginx/set_upstream'),
        headers={'User-agent': settings.USER_AGENT},
        proxies=get_proxy_api_dict(),
        timeout=30,
        json=data
    )
    return answer


def del_list_upstreams_from_api(data: dict) -> object:
    answer = requests.post(
        url_normalize(api_url + '/nginx/set_upstream'),
        headers={'User-agent': settings.USER_AGENT},
        proxies=get_proxy_api_dict(),
        timeout=30,
        json=data
    )

    return answer


def get_list_upstreams_from_api(list_key_for_requests: list) -> dict:
    try:
        answer = requests.post(
            url_normalize(api_url + '/nginx/get_upstream'),
            headers={'User-agent': settings.USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            data={'key': list_key_for_requests[0].key},
        )
        if answer.ok:
            answer = json.loads(answer.content.decode())
            return answer
    except Exception as error:
        logger.error(f'An error occurred error when get /nginx/get_upstream {error}')


@login_required
@time_of_function
def main_upstream(request):
    keys = LicKeys.objects.filter(user__id=request.user.id).all()
    lic_keys, checked_ids = check_keys(keys, check_waf_id=True)
    list_key_for_requests = [i for i in lic_keys if i.tarif == 'Enterprise']
    update = request.GET.get('update')
    delete = request.GET.get('del')
    if delete:
        dict_for_api = {
            "key": list_key_for_requests[0].key,
            "del": {'name': delete}
        }

        del_list_upstreams_from_api(data=dict_for_api)
    if update:
        values_dict = json.loads(request.GET['data'])
        dict_for_api = {
            "key": list_key_for_requests[0].key,
            "set": values_dict
        }
        answer = set_list_upstreams_from_api(data=dict_for_api)
        answer = json.loads(answer.content)
        if answer['status'] != "success":
            return JsonResponse(data={'messages': answer['description'], 'status': 'error'}, status=255)

    answer = get_list_upstreams_from_api(list_key_for_requests)

    if not answer['status'] != 'none':
        answer = {'configuration': []}
    context = {
        'list_upstreams': answer['configuration']

    }
    return render(request, 'nginx/main_upstreams.html', context)


@login_required
@time_of_function
def summary_nginx(request):
    keys = LicKeys.objects.filter(user__id=request.user.id).all()
    keys_list = []
    lic_keys, checked_ids = check_keys(keys, check_waf_id=True)

    for key in lic_keys:
        waf_id = get_waf_id(key.key)
        if waf_id is not None:
            key.waf_id = waf_id
            keys_list.append(key)

    for key in keys_list:
        get_conf = get_nginx_json(key.key)
        if get_conf:
            for i in get_conf:
                if i in list_type_list_keys:
                    get_conf[i] = ', '.join(get_conf[i])

            key.conf = get_conf
    keys_list = sorted(keys_list, key=operator.attrgetter('description'))

    context = {'keys_list': keys_list, 'home': settings.HOME_PAGE}
    return render(request, 'nginx/nginx_index.html', context)


def pars_locations(locations_list: list, var_location=None) -> None:
    """pars array location for vhost"""
    list_keys = ['pass_type', 'index', 'path', 'autoindex',
                 'limit_req', 'limit_req_status',
                 "proxy_http_version", "proxy_intercept_errors",
                 "proxy_pass", "proxy_connect_timeout", "memcached_pass", 'proxy_intercept_errors']
    for value in locations_list['locations']:
        for key in list_keys:
            if key in value and value[key] != 'None':

                if key == 'proxy_intercept_errors':
                    value[key] = str(value[key]).replace('[\'', '').replace('\']', '').split()
                else:
                    value[key] = str(value[key]).split()
                value['name_vhost'] = [locations_list['name']]

        if var_location is not None:
            var_location.append(value)


def pars_answer_main_vhost(answer: dict, var_location=None) -> None:
    """pars array from api"""
    list_keys = ['listen', 'index', 'server_name', 'root', 'locations']
    for i in answer['configuration']:

        for key in list_keys:
            if i[key]:
                if key == 'locations':
                    if var_location is not None:
                        pars_locations(locations_list=i, var_location=var_location)
                    else:
                        pars_locations(locations_list=i)
                else:
                    i[key] = i[key].split()


def check_key_in_locations_and_append_in_answer(dict_args: dict):
    if "memcached_pass" in dict_args['loc']:
        for i in dict_args['answer_upstreams']:
            if i['name'] == dict_args['loc']["memcached_pass"][0]:
                if i not in dict_args['val']:
                    dict_args['val'].append(i)
    if "proxy_pass" in dict_args['loc']:
        for i in dict_args['answer_upstreams']:
            if "proxy_pass" in dict_args['loc']:
                if len(dict_args['loc']["proxy_pass"]) > 1:
                    if i['name'] == dict_args['loc']["proxy_pass"][0].replace('http://', ''):
                        if i not in dict_args['val']:
                            dict_args['val'].append(i)


def append_upstreams_in_vhost(answer: dict, answer_upstreams: dict) -> None:
    """ merge list upstreams and answer list"""
    for val in answer['configuration']:
        val['upstreams'] = []

    for val in answer['configuration']:
        if val['locations']:
            for loc in val['locations']:
                dict_args = {
                    'answer_upstreams': answer_upstreams['configuration'],
                    'val': val['upstreams'],
                    'loc': loc
                }
                check_key_in_locations_and_append_in_answer(dict_args)


def request_in_api_vhost(list_key_for_requests):
    answer = api_vhost_only_query_func(list_key_for_requests)
    return answer, list_key_for_requests


def vhost_logick_main_and_nginx_vhost(request, list_key_for_requests) -> tuple:
    answer, list_key_for_requests = request_in_api_vhost(list_key_for_requests)
    if answer.ok:
        answer = json.loads(answer.content.decode())
        answer_ssl = get_from_api_list_ssl(list_key_for_requests)
        answer_ssl = decode_base64_ssl(answer_ssl)
        if 'configuration' not in answer_ssl:
            answer_ssl = {'configuration': [], 'status': 'none'}
        if answer['status'] == 'none':
            answer = {'configuration': []}
        for vhost in answer['configuration']:
            for ssl in answer_ssl['configuration']:
                if vhost['certificate_name'] == ssl['name']:
                    vhost['i'] = ssl

        pars_answer_main_vhost(answer)
        answer_upstreams = get_list_upstreams_from_api(list_key_for_requests)
        if answer['configuration'] and 'configuration' in answer_upstreams:
            append_upstreams_in_vhost(answer, answer_upstreams)
    return answer, list_key_for_requests


@login_required
@time_of_function
def main_vhosts(request):
    keys = LicKeys.objects.filter(user__id=request.user.id).all()
    lic_keys, checked_ids = check_keys(keys, check_waf_id=True)
    list_key_for_requests = [i for i in lic_keys if i.tarif == 'Enterprise']
    del_vhost = request.GET.get('del')
    if del_vhost:
        data = {
            'key': list_key_for_requests[0].key,
            'del': {'name': del_vhost}

        }
        requests.post(
            url_normalize(api_url + '/nginx/set_vhost'),
            headers={'User-agent': settings.USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json=data
        )
    if 'insert' in request.POST:
        add_vhost(request, list_key_for_requests)
    answer, lic_keys = vhost_logick_main_and_nginx_vhost(request, list_key_for_requests)
    if not answer['configuration']:
        answer = {'configuration': []}

    context = {
        'list_vhost': answer['configuration']
    }
    return render(request, 'nginx/main_vhosts.html', context)


def set_main_location_in_api(data: dict):
    answer = requests.post(
        url_normalize(api_url + '/nginx/set_vhost'),
        headers={'User-agent': settings.USER_AGENT},
        proxies=get_proxy_api_dict(),
        timeout=30,
        json=data
    )
    return answer


def check_info_location(changes_location, i, list_for_api):
    if 'proxy_http_version' in changes_location:
        changes_location['proxy_http_version'] = \
            float(changes_location['proxy_http_version'])
    if 'proxy_intercept_errors' in changes_location:
        changes_location['proxy_intercept_errors'] = \
            bool(changes_location['proxy_intercept_errors'])
    i['locations'].append(changes_location)
    list_for_api.append(i)


@login_required
@time_of_function
def main_locations(request):
    lic_keys, list_key_for_requests, list_for_template = get_lickeys(request)

    if 'del_data' in request.POST:
        list_for_api = []
        del_location = json.loads(request.POST['del_data'])
        answer_vhosts = api_vhost_only_query_func(list_key_for_requests)
        answer_vhost = json.loads(answer_vhosts.content)
        for i in answer_vhost['configuration']:
            if i['name'] == del_location['vhost']:
                for loc in i['locations']:
                    if 'location' in loc:
                        if loc['location'] == del_location['del']:
                            i['locations'].remove(loc)
                            list_for_api.append(i)

        for i in list_for_api[0]['locations']:
            if 'vhost_name' in i:
                i.pop('vhost_name', None)
        for key in list(list_for_api[0]):
            if not list_for_api[0][key]:
                list_for_api[0].pop(key, None)

        data_for_api = {
            'key': list_key_for_requests[0].key,
            "set": list_for_api[0]
        }

        answer = set_main_location_in_api(data=data_for_api)
        answer = json.loads(answer.content)

        if answer['status'] != 'success':
            return JsonResponse(data={'messages': answer['description'], 'status': 'error'}, status=255)

    if 'data' in request.POST:
        changes_location = json.loads(request.POST['data'])
        answer_vhosts = api_vhost_only_query_func(list_key_for_requests)
        answer_vhost = json.loads(answer_vhosts.content)
        list_for_api = []

        changes_to_format_add_header_and_error_page(changes_location)

        for i in answer_vhost['configuration']:
            if i['name'] == changes_location['vhost_name']:
                if i['locations']:
                    if 'insert' in request.POST:
                        check_info_location(changes_location, i, list_for_api)
                    else:
                        for loc in i['locations']:
                            if loc['location'] == changes_location['location']:
                                i['locations'].remove(loc)
                                check_info_location(changes_location, i, list_for_api)

                else:
                    i['locations'] = []
                    i['locations'].append(changes_location)
                    list_for_api.append(i)
        for i in list_for_api[0]['locations']:
            if 'vhost_name' in i:
                i.pop('vhost_name', None)
        for key in list(list_for_api[0]):
            if not list_for_api[0][key]:
                list_for_api[0].pop(key, None)

        data_for_api = {
            'key': list_key_for_requests[0].key,
            "set": list_for_api[0]
        }

        answer = set_main_location_in_api(data=data_for_api)
        answer = json.loads(answer.content)

        if answer['status'] != "success":
            return JsonResponse(data={'messages': answer['description'], 'status': 'error'}, status=255)

    try:
        answer = requests.post(
            url_normalize(api_url + '/nginx/get_vhost'),
            headers={'User-agent': settings.USER_AGENT},
            proxies=get_proxy_api_dict(),
            timeout=30,
            data={'key': list_key_for_requests[0].key},
        )
        if answer.ok:
            answer = json.loads(answer.content.decode())
            list_location = []
            if answer['status'] != 'none':
                pars_answer_main_vhost(answer, var_location=list_location)

    except Exception as error:
        logger.error(f'An error occurred error when deleting {error}')
    context = {
        'list_location': list_location
    }
    return render(request, 'nginx/main_location.html', context)


def return_list_locations_for_vhost_location(answer: dict) -> list:
    """ def chek locations and if"""
    list_locations = []
    if answer['locations']:
        for i in answer['locations']:
            if 'location' in i:
                list_locations.append(i['location'])

    return list_locations


def str_to_dict_add_header_and_error_page(data: dict):
    local_dict = {}
    list_value = data['values'].split(',')

    for i in list_value:
        list_values = i.split(':')
        if data['tag'] == 'add_header':
            dict_values = {list_values[0].replace(' ', ''): list_values[1].lstrip().rstrip()}
        else:
            dict_values = {list_values[0].lstrip().rstrip(): list_values[1].lstrip().rstrip()}

        local_dict.update(dict_values)

    return local_dict


def add_column_in_vhost_add_header_and_error_page(data: dict, answer_set_param: dict) -> None:
    if 'tag' in data:
        if data['tag'] == 'add_header' or data['tag'] == 'error_page':

            if data['values']:

                local_dict = str_to_dict_add_header_and_error_page(data)
                answer_set_param['configuration'][0][data['tag']] = local_dict

            else:
                answer_set_param['configuration'][0][data['tag']] = None


def set_param_vhost(request, list_key_for_requests) -> None:
    try:
        answer_api, list_key_for_requests = request_in_api_vhost(list_key_for_requests)
        answer_set_param = json.loads(answer_api.content.decode())
        data = json.loads(request.POST['data'])
        answer_set_param['configuration'] = [value for value in answer_set_param['configuration']
                                             if value['name'] == data['vhost']]

        if 'location' in data:
            for i in list(answer_set_param['configuration'][0][data['tag']]):
                if i['location'] == data['location']:

                    if data['key'] == 'add_header' or data['key'] == 'error_page' or data['key'] == 'proxy_set_header':

                        i[data['key']] = str_to_dict_add_header_and_error_page(data)

                    else:
                        i[data['key']] = data['values']
        elif 'param_ssl' in data:
            if not data['value_ssl']:
                data['value_ssl'] = 'null'
            if data['param_ssl'] == 'ssl_session_timeout':
                data['value_ssl'] = str(data['value_ssl'])
            answer_set_param['configuration'][0][data['param_ssl']] = data['value_ssl']
        else:
            answer_set_param['configuration'][0][data['tag']] = data['values']

        for key in list(answer_set_param['configuration'][0]):
            add_column_in_vhost_add_header_and_error_page(data, answer_set_param)
            if not answer_set_param['configuration'][0][key]:
                del answer_set_param['configuration'][0][key]

        answer = requests.post(
            url_normalize(api_url + '/nginx/set_vhost'),
            headers={'User-agent': settings.USER_AGENT, 'Content-type': 'application/json'},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json={'key': list_key_for_requests[0].key, 'set': answer_set_param['configuration'][0]},
        )
    except Exception as error:
        logger.error(f'An error occurred error when set_param_vhost {error}')


def vhost_nginx_if_answer(answer: list, request: object) -> tuple:
    list_vhost_fro_dropdown_menu = [value['name'] for value in answer['configuration']]

    vhost_attr = request.GET.get('vhost')
    if not vhost_attr:
        vhost_attr = request.POST.get('vhost')

    if vhost_attr:
        answer['configuration'] = [value for value in answer['configuration']
                                   if value['name'] == vhost_attr]
    else:
        answer['configuration'] = answer['configuration']

    list_locations = return_list_locations_for_vhost_location(answer['configuration'][0])
    location_choose = request.GET.get('location')
    if not location_choose:
        if answer['configuration'][0]['locations']:
            location_choose = answer['configuration'][0]['locations'][0]['location']
    return list_vhost_fro_dropdown_menu, list_locations, location_choose


def changes_to_format_add_header_and_error_page(data: dict):
    if 'add_header' in data:
        if data['add_header']:
            local_dict = {}
            list_value = data['add_header'].split(',')
            for i in list_value:
                list_values = i.split(':')
                dict_values = {list_values[0].replace(' ', ''): list_values[1].lstrip().rstrip()}
                local_dict.update(dict_values)
            data['add_header'] = local_dict
    if 'error_page' in data:
        if data['error_page']:
            local_dict = {}
            list_value = data['error_page'].split(',')
            for i in list_value:
                list_values = i.split(':')
                dict_values = {list_values[0].lstrip().rstrip(): list_values[1].lstrip().rstrip()}
                local_dict.update(dict_values)
            data['error_page'] = local_dict
    if 'proxy_set_header' in data:
        if data['proxy_set_header']:
            local_dict = {}
            list_value = data['proxy_set_header'].split(',')
            for i in list_value:
                list_values = i.split(':')
                dict_values = {list_values[0].lstrip().rstrip(): list_values[1].lstrip().rstrip()}
                local_dict.update(dict_values)
            data['proxy_set_header'] = local_dict


def add_vhost(request, list_key_for_requests):
    data = json.loads(request.POST['insert'])
    changes_to_format_add_header_and_error_page(data)
    for i in list(data):
        if not data[i]:
            del data[i]

    answer = requests.post(
        url_normalize(api_url + '/nginx/set_vhost'),
        headers={'User-agent': settings.USER_AGENT, 'Content-type': 'application/json'},
        proxies=get_proxy_api_dict(),
        timeout=30,
        json={'key': list_key_for_requests[0].key, 'set': data},
    )

    return answer


def changes_ssl_from_vhosts(request, list_key_for_requests, ssl):
    answer_api, list_key_for_requests = request_in_api_vhost(list_key_for_requests)
    answer_set_param = json.loads(answer_api.content.decode())

    list_for_api = []
    if 'configuration' in answer_set_param:
        for i in list(answer_set_param['configuration']):
            if i['name'] == ssl['name']:
                for ssl_val in list(ssl):
                    i[ssl_val] = ssl[ssl_val]
                list_for_api.append(i)
    for key in list(list_for_api[0]):
        if not list_for_api[0][key]:
            list_for_api[0].pop(key, None)

    answer = requests.post(
        url_normalize(api_url + '/nginx/set_vhost'),
        headers={'User-agent': settings.USER_AGENT, 'Content-type': 'application/json'},
        proxies=get_proxy_api_dict(),
        timeout=30,
        json={'key': list_key_for_requests[0].key, 'set': list_for_api[0]},
    )


def delete_cert_of_vhost(request, cert_delete, list_key_for_requests):
    answer_list = api_vhost_only_query_func(list_key_for_requests)
    answer = json.loads(answer_list.content.decode())
    list_for_api = []
    if 'configuration' in answer:
        for i in answer['configuration']:
            if i['name'] == cert_delete['name']:
                i['certificate_name'] = None
                list_for_api.append(i)

    requests.post(
        url_normalize(api_url + '/nginx/set_vhost'),
        headers={'User-agent': settings.USER_AGENT, 'Content-type': 'application/json'},
        proxies=get_proxy_api_dict(),
        timeout=30,
        json={'key': list_key_for_requests[0].key, 'set': list_for_api[0]},
    )


def delete_dh_param_of_vhost(dh_param_set_query: dict, list_key_for_requests):
    answer_list = api_vhost_only_query_func(list_key_for_requests)
    answer = json.loads(answer_list.content.decode())

    list_for_api = []
    if 'configuration' in answer:
        for i in answer['configuration']:

            if i['name'] == dh_param_set_query['name']:
                i['dhparam_name'] = None
                list_for_api.append(i)

        requests.post(
            url_normalize(api_url + '/nginx/set_vhost'),
            headers={'User-agent': settings.USER_AGENT, 'Content-type': 'application/json'},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json={'key': list_key_for_requests[0].key, 'set': list_for_api[0]},
        )


def set_param_dh_param(dh_param_set_query: dict, list_key_for_requests):
    answer_list = api_vhost_only_query_func(list_key_for_requests)

    answer = json.loads(answer_list.content.decode())
    list_for_api = []
    if 'configuration' in answer:
        for i in answer['configuration']:
            if i['name'] == dh_param_set_query['name']:
                i['dhparam_name'] = dh_param_set_query['dhparam_name']
                list_for_api.append(i)

        requests.post(
            url_normalize(api_url + '/nginx/set_vhost'),
            headers={'User-agent': settings.USER_AGENT, 'Content-type': 'application/json'},
            proxies=get_proxy_api_dict(),
            timeout=30,
            json={'key': list_key_for_requests[0].key, 'set': list_for_api[0]},
        )


@login_required
@time_of_function
def vhost_nginx_with_tab_content(request, var=None):
    lic_keys, list_key_for_requests, list_for_template = get_lickeys(request)
    dh_param = get_dhparam_for_templates(list_key_for_requests)
    answer_ssl = get_from_api_list_ssl(list_key_for_requests)
    answer_ssl = decode_base64_ssl(answer_ssl)

    if 'configuration' not in answer_ssl:
        answer_ssl = {'configuration': [], 'status': 'none'}

    if 'configuration' not in dh_param:
        dh_param = {'configuration': [], 'status': 'none'}

    cert_delete = request.POST.get('ssl_delete')
    if cert_delete:
        delete_cert_of_vhost(request, json.loads(cert_delete), list_key_for_requests)

    cert = request.POST.get('ssl')
    if cert:
        changes_ssl_from_vhosts(request, list_key_for_requests, ssl=json.loads(cert))

    dh_param_set = request.POST.get('dh_param')
    if dh_param_set:
        dh_param_set_query = json.loads(dh_param_set)
        set_param_dh_param(dh_param_set_query, list_key_for_requests)

    dh_param_delete = request.POST.get('dh_param_delete')

    if dh_param_delete:
        dh_param_delete_query = json.loads(dh_param_delete)
        delete_dh_param_of_vhost(dh_param_delete_query, list_key_for_requests)

    set_param = request.GET.get('set_param')
    if set_param:
        set_param_vhost(request, list_key_for_requests)

    answer, lic_keys = vhost_logick_main_and_nginx_vhost(request, list_key_for_requests)
    if not answer['configuration']:
        answer = {'configuration': [{'name': ''}]}
        list_vhost_fro_dropdown_menu = []
        list_locations = []
        location_choose = ''
        answer_ssl['configuration'] = []
    else:

        list_vhost_fro_dropdown_menu, list_locations, location_choose = vhost_nginx_if_answer(answer, request)
    context = {
        'keys_list': lic_keys,
        'home': settings.HOME_PAGE,
        'list_vhost': answer['configuration'][0],
        'dropdown_menu': list_vhost_fro_dropdown_menu,
        'vhost_menu': answer['configuration'][0]['name'],
        'list_locations': list_locations,
        'location_choose': location_choose,
        'list_certificate': answer_ssl['configuration'],
        'dh_param': dh_param['configuration'],
        'list_key_for_requests': list_for_template,
        'license_key_id': list_key_for_requests
    }
    if var:

        return context
    else:
        return render(request, 'nginx/vhost_nginx_with_tab_content.html', context)


@login_required
@time_of_function
def vhost_nginx(request):
    context = vhost_nginx_with_tab_content(request, var=True)
    return render(request, 'nginx/nginx_vhost.html', context)


def get_main_ngin_param(list_key_for_requests: list) -> object:
    answer = requests.post(
        url_normalize(api_url + '/nginx/get_nginx'),
        headers={'User-agent': settings.USER_AGENT},
        proxies=get_proxy_api_dict(),
        timeout=30,
        data={'key': list_key_for_requests[0].key},
    )
    return answer


def set_main_ngin_param(data: dict) -> object:
    answer = requests.post(
        url_normalize(api_url + '/nginx/set_nginx'),
        headers={'User-agent': settings.USER_AGENT},
        proxies=get_proxy_api_dict(),
        timeout=30,
        json=data,
    )
    return answer


def tab_main_nginx(request, var=None):
    lic_keys, list_key_for_requests, list_for_template = get_lickeys(request)
    if 'data' in request.POST:
        data_for_update = json.loads(request.POST['data'])
        get_answer = get_main_ngin_param(list_key_for_requests)
        answer = json.loads(get_answer.content.decode())

        for value in list(data_for_update):
            if not data_for_update[value]:
                data_for_update[value] = None

        for i in answer['configuration'][0]:
            if i == data_for_update['param_main']:
                if data_for_update['param_main'] == 'set_real_ip_from':
                    if data_for_update['value_main']:
                        answer['configuration'][0][i] = data_for_update['value_main'].replace('\"', '').split()
                    else:
                        answer['configuration'][0][i] = None
                elif data_for_update['param_main'] == "limit_req_zone":
                    if data_for_update['value_main']:
                        answer['configuration'][0][i] = data_for_update['value_main'].replace('\"', '').split(',')
                    else:
                        answer['configuration'][0][i] = None
                elif data_for_update['param_main'] == 'client_max_body_size' or \
                        data_for_update['param_main'] == "resolver_timeout":
                    if data_for_update['value_main']:
                        answer['configuration'][0][i] = str(data_for_update['value_main'])
                    else:
                        answer['configuration'][0][i] = None
                else:
                    answer['configuration'][0][i] = data_for_update['value_main']

        data = {
            'key': list_key_for_requests[0].key,
            'set': answer['configuration'][0]
        }

        answer = set_main_ngin_param(data)
        answer = json.loads(answer.content)

        if answer['status'] != "success":
            return JsonResponse(data={'messages': answer['description'], 'status': 'error'}, status=255)

    answer = get_main_ngin_param(list_key_for_requests)
    answer = json.loads(answer.content.decode())
    if 'configuration' in answer:
        if len(answer["configuration"]) >= 1 and 'resolver' in answer["configuration"][0]:
            if answer["configuration"][0]['resolver']:
                answer["configuration"][0]['resolver'] = answer["configuration"][0]['resolver'].split()
    else:
        answer = {'configuration': [{'name': ''}]}
    context = {
        'home': settings.HOME_PAGE,
        'keys_list': lic_keys,
        'main_conf': answer["configuration"],
        'list_key_for_requests': list_for_template,
        'license_key_id': list_key_for_requests
    }
    if var:
        return context
    else:
        return render(request, 'nginx/main_nginx_include_main.html', context)


@login_required
@time_of_function
def main_nginx(request):
    context = tab_main_nginx(request, var=True)
    return render(request, 'nginx/nginx_main.html', context)


@require_POST
@login_required
@time_of_function
def set_settings_conf(request):
    selected_key = request.POST.get('key')
    hashed_key = LicKeys.objects.get(id=selected_key).key
    data = request.POST.get('data')
    table = request.POST.get('table')
    try:
        data = json.loads(data)
    except json.JSONDecodeError:
        messages.error(request, pgettext('messages', "Invalid format"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)
        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)

    if table == 'nginx':
        r = set_nginx_conf(hashed_key, data)
        if not r[0]:
            messages.error(request, pgettext('messages', r[1]))
            storage = messages.get_messages(request)
            list_messages = get_list_messages(storage)

            return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)
    elif table == 'nginx_vhost':
        r = set_nginx_conf(hashed_key, data)
        if not r[0]:
            messages.error(request, pgettext('messages', r[1]))
            storage = messages.get_messages(request)
            list_messages = get_list_messages(storage)

            return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)
    elif table == 'nginx_ssl':
        r = set_nginx_conf(hashed_key, data)
        if not r[0]:
            messages.error(request, pgettext('messages', r[1]))
            storage = messages.get_messages(request)
            list_messages = get_list_messages(storage)

            return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)
    elif table == 'nginx_upstream':
        r = set_nginx_conf(hashed_key, data)
        if not r[0]:
            messages.error(request, pgettext('messages', r[1]))
            storage = messages.get_messages(request)
            list_messages = get_list_messages(storage)

            return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)
    elif table == 'nginx_location':
        r = set_nginx_conf(hashed_key, data)
        if not r[0]:
            messages.error(request, pgettext('messages', r[1]))
            storage = messages.get_messages(request)
            list_messages = get_list_messages(storage)

            return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)
    else:
        messages.error(request, pgettext('messages', "Invalid table"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)

        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)

    return JsonResponse(data={"status": "success"}, status=200)


@require_POST
@login_required
@time_of_function
def get_settings_info(request):
    settings_info_info = {
        'listen': _('listen'),
        'index_ssl': _('index_ssl'),
        'server_name': _('server name'),
        'root': _('root'),
        'dhparam_name': _('dhparam name'),
        'certificate_name': _('certificate name'),
        'certificate': _('certificate'),
        'certificate_key': _('certificate key'),
        'ssl_protocols': _('ssl_protocols'),
        'ssl_session_timeout': _('ssl_session_timeout'),
        'ssl_ciphers': _('ssl_ciphers'),
        'locations_ssl': _('locations_ssl'),
        'limit_req_zone': _('limit_req_zone'),
        'client_max_body_size': _('client_max_body_size'),
        'real_ip_header': _('real_ip_header'),
        'set_real_ip_from': _('set_real_ip_from'),
        'resolver_timeout': _('resolver_timeout'),
        'resolver': _('resolver'),

    }

    return JsonResponse(settings_info_info)


@require_POST
@login_required
@time_of_function
def dh_param_nginx(request):
    context = {
        'listen': pgettext('nginx', 'listen info'),

    }

    return render(request, 'nginx/main_dh_param_include_dh_param.html', context)


@require_POST
@login_required
@time_of_function
def get_data_json(request):
    settings_info_info = {
        'listen': pgettext('nginx', 'listen info'),

    }

    return JsonResponse(settings_info_info)
