from dataclasses import dataclass
from django.conf import settings


@dataclass()
class AttackCounter:
    description: str
    numbers_of_attacks: int


def make_attack_type_counters(attacks_logs) -> list:
    """ Returns a list of objects that are needed to fill in the beginning of the report """
    res = make_counter(attacks_logs)
    return res


def make_other_counter(attacks_logs) -> AttackCounter:
    """ counting attacks not included in various categories """
    exclude_numbers = [bt for bt in settings.TITLE_BT_COUNTERS.keys()]
    exclude_descriptions = list(settings.TITLE_BT_DESCRIPTION_COUNTER.keys())
    other_count = attacks_logs.exclude(bt__in=exclude_numbers).exclude(
        description__iregex=r'(' + '|'.join(exclude_descriptions) + ')').count()
    return AttackCounter(
        description='Other',
        numbers_of_attacks=other_count
    )


def make_counter_list_by_bt(attacks_logs) -> list:
    res = []
    bt_list = [bt for bt in settings.TITLE_BT_COUNTERS.keys()]
    for bt in bt_list:
        res.append(
            AttackCounter(
                numbers_of_attacks=attacks_logs.filter(bt=bt).count(),
                description=settings.TITLE_BT_COUNTERS[bt],
            )
        )
    return res


def make_counter(attacks_logs) -> list:
    count_xss_atacks = len([elem for elem in attacks_logs if elem['description'] == 'XSS'])
    count_SQLi_atacks = len([elem for elem in attacks_logs if elem['description'] == 'SQLi'])
    count_Brute_force_atacks = len([elem for elem in attacks_logs if elem['description'] == 'Brute-force'])
    count_ddos_atacks = len([elem for elem in attacks_logs if elem['description'] == 'Ddos'])
    other = len([elem for elem in attacks_logs if elem['description'] == 'Other'])

    return [AttackCounter(numbers_of_attacks=count_xss_atacks, description='XSS'),
            AttackCounter(numbers_of_attacks=count_SQLi_atacks, description='SQL Injection'),
            AttackCounter(numbers_of_attacks=count_Brute_force_atacks, description='Brute-force'),
            AttackCounter(numbers_of_attacks=count_ddos_atacks, description='DDoS'),
            AttackCounter(numbers_of_attacks=other, description='Other'),

            ]


def make_counter_list_by_description(attacks_logs) -> list:
    res = []
    for db_description, report_description in settings.TITLE_BT_DESCRIPTION_COUNTER.items():
        res.append(
            AttackCounter(
                numbers_of_attacks=attacks_logs.filter(description__icontains=db_description).count(),
                description=report_description,
            )
        )
        return res


def make_extra_counters(attacks_logs) -> list:
    res = []
    for description, bt_list in settings.TITLE_BT_EXTRA_COUNTERS.items():
        res.append(
            AttackCounter(
                description=description,
                numbers_of_attacks=attacks_logs.filter(bt__in=bt_list).count()
            )

        )

    return res
