""" sends pdf mail to user mail """
import logging
from pathlib import Path

from django.conf import settings
from django.db import close_old_connections
from django.utils.crypto import get_random_string
from django.utils.translation import gettext as _

from utils.mail import send_mail_from_cabinet

logger = logging.getLogger(__name__)


def send_pdf_link_to_mail(user_mail, host, pdf_path: Path, period: str):
    close_old_connections()  # to prevent some errors with multiproc
    link = _generate_file_link(host, pdf_path)
    subject = _('pdf_gen: mail subject') + ' ' + period
    text = _('pdf_gen: send link mail text').replace('_period', period).replace('_link', link)
    send_mail_from_cabinet(
        send_to=user_mail,
        subject=subject,
        text=text,
        logger=logger,
    )


def _generate_file_link(host, filepath: Path):
    from django.core.cache import cache
    key = 'pdf-' + get_random_string(12)
    cache.set(key, filepath, 24 * 60 * 60)
    link = '/'.join([host, settings.HOME_PAGE, 'report', key]).replace('//', '/')
    if not link.startswith('http'):
        link = 'http://' + link
    return link
