from functools import wraps

from django.core.cache import cache

from main.common import get_client_ip


def rate_limit(limit: int = 6, timeout: int = 30):
    """The decorator sets a limit for requests to a specific function from a specific ip.
     To use:
     check if there is a request.rate_limit_error in the function, if True - display an error """

    def actual_decorator(func):
        @wraps(func)
        def wrapper(request, *args, **kwargs):
            request.rate_limit_error = False

            if request.method == 'POST':
                ip = get_client_ip(request)
                rate_limit_key = f"rate_limit_{ip}_{func.__name__}"
                rate_limit = cache.get(rate_limit_key)
                if rate_limit is None:
                    cache.set(rate_limit_key, 1, timeout)
                elif rate_limit >= limit:
                    request.rate_limit_error = True
                else:
                    cache.incr(rate_limit_key)
            func_result = func(request, *args, **kwargs)
            return func_result

        return wrapper

    return actual_decorator
