import psycopg2

from django.db import connections
from time import sleep


def check_db(log):
    try:
        s = connections['waf'].introspection.table_names()
        return True if 'mt_export' in s and 'mt_main' in s and 'openapi_schemas' in s else False
    except Exception as e:
        if log is not None:
            log.error('An error occurred while checking the database: {}'.format(e))
        return False


def init_db_connection(db_name, db_user, db_pass, db_host, db_port, log):
    while True:
        try:

            connect = psycopg2.connect(
                database=db_name,
                user=db_user,
                password=db_pass,
                host=db_host,
                port=db_port,
                connect_timeout=10
            )
            connect.set_session(autocommit=True)
            return connect

        except Exception as e:
            log.error('An error occurred while initializing the connection to the database {}: {}'.format(db_name, e))
            sleep(5)


def close_db_connection(cursor, connect, log):
    try:
        cursor.close()
        connect.close()
    except Exception as e:
        log.error('An error occurred while closing the connection to the database: {}'.format(e))
